@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -45,9 +45,9 @@ using namespace std;
 /** Instances *must* be allocated/freed using methods herein, because the C++ declaration
     represents only the header of a much larger object in memory. */
 class TextSlice {
-    //! Pointer to one past last character in slice
+    //! Pointer to one past last character in sequence
     char* logical_end;
-    //! Pionter to one past last available byte in slice.
+    //! Pointer to one past last available byte in sequence.
     char* physical_end;
 public:
     //! Allocate a TextSlice object that can hold up to max_size characters.
@@ -60,7 +60,7 @@ public:
     }
     //! Free a TextSlice object 
     void free() {
-        tbb::tbb_allocator<char>().deallocate((char*)this,size());
+        tbb::tbb_allocator<char>().deallocate((char*)this,sizeof(TextSlice)+(physical_end-begin())+1);
     } 
     //! Pointer to beginning of sequence
     char* begin() {return (char*)(this+1);}
@@ -68,14 +68,14 @@ public:
     char* end() {return logical_end;}
     //! Length of sequence
     size_t size() const {return logical_end-(char*)(this+1);}
-    //! Maximum number of characters that can be appended to sequkence
+    //! Maximum number of characters that can be appended to sequence
     size_t avail() const {return physical_end-logical_end;}
     //! Append sequence [first,last) to this sequence.
     void append( char* first, char* last ) {
         memcpy( logical_end, first, last-first );
         logical_end += last-first;
     }
-    //! Set end to given value.
+    //! Set end() to given value.
     void set_end( char* p ) {logical_end=p;}
 };
 
@@ -94,7 +94,7 @@ private:
 };
 
 MyInputFilter::MyInputFilter( FILE* input_file_ ) : 
-    filter(/*is_serial=*/true),
+    filter(serial_in_order),
     input_file(input_file_),
     next_slice( TextSlice::allocate( MAX_CHAR_PER_INPUT_SLICE ) )
 { 
@@ -135,16 +135,16 @@ public:
 };
 
 MyTransformFilter::MyTransformFilter() : 
-    tbb::filter(/*ordered=*/false) 
+    tbb::filter(parallel) 
 {}  
 
 /*override*/void* MyTransformFilter::operator()( void* item ) {
     TextSlice& input = *static_cast<TextSlice*>(item);
-    // Add terminating NULL so that strtol works right even if number is at end of the input.
+    // Add terminating null so that strtol works right even if number is at end of the input.
     *input.end() = '\0';
     char* p = input.begin();
-    TextSlice& output = *TextSlice::allocate( 2*MAX_CHAR_PER_INPUT_SLICE );
-    char* q = output.begin();
+    TextSlice& out = *TextSlice::allocate( 2*MAX_CHAR_PER_INPUT_SLICE );
+    char* q = out.begin();
     for(;;) {
         while( p<input.end() && !isdigit(*p) ) 
             *q++ = *p++; 
@@ -158,9 +158,9 @@ MyTransformFilter::MyTransformFilter() :
         sprintf(q,"%ld",y);
         q = strchr(q,0);
     }
-    output.set_end(q);
+    out.set_end(q);
     input.free();
-    return &output;
+    return &out;
 }
          
 //! Filter that writes each buffer to a file.
@@ -172,19 +172,19 @@ public:
 };
 
 MyOutputFilter::MyOutputFilter( FILE* output_file ) : 
-    tbb::filter(/*is_serial=*/true),
+    tbb::filter(serial_in_order),
     my_output_file(output_file)
 {
 }
 
 void* MyOutputFilter::operator()( void* item ) {
-    TextSlice& output = *static_cast<TextSlice*>(item);
-    size_t n = fwrite( output.begin(), 1, output.size(), my_output_file );
-    if( n!=output.size() ) {
-        fprintf(stderr,"Can't write into %s file\n", OutputFileName);
+    TextSlice& out = *static_cast<TextSlice*>(item);
+    size_t n = fwrite( out.begin(), 1, out.size(), my_output_file );
+    if( n!=out.size() ) {
+        fprintf(stderr,"Can't write into file '%s'\n", OutputFileName);
         exit(1);
     }
-    output.free();
+    out.free();
     return NULL;
 }
 
@@ -236,7 +236,7 @@ int run_pipeline( int nthreads )
     MyInputFilter input_filter( input_file );
     pipeline.add_filter( input_filter );
 
-    // Create capitalization stage and add it to the pipeline
+    // Create squaring stage and add it to the pipeline
     MyTransformFilter transform_filter; 
     pipeline.add_filter( transform_filter );
 
@@ -251,9 +251,6 @@ int run_pipeline( int nthreads )
     pipeline.run( nthreads*4 );
     tbb::tick_count t1 = tbb::tick_count::now();
 
-    // Remove filters from pipeline before they are implicitly destroyed.
-    pipeline.clear(); 
-
     fclose( output_file );
     fclose( input_file );
 
