@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -29,40 +29,59 @@
 #ifndef __TBB_concurrent_hash_map_H
 #define __TBB_concurrent_hash_map_H
 
-#include <stdexcept>
+#include "tbb_stddef.h"
+
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+    #pragma warning (push)
+    #pragma warning (disable: 4530)
+#endif
+
 #include <iterator>
 #include <utility>      // Need std::pair
 #include <cstring>      // Need std::memset
-#include <string>
-#include "tbb_stddef.h"
+
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    #pragma warning (pop)
+#endif
+
 #include "cache_aligned_allocator.h"
 #include "tbb_allocator.h"
 #include "spin_rw_mutex.h"
 #include "atomic.h"
 #include "aligned_space.h"
-#if TBB_USE_PERFORMANCE_WARNINGS
+#include "tbb_exception.h"
+#include "tbb_profiling.h"
+#include "_concurrent_unordered_internal.h" // Need tbb_hasher
+#if TBB_USE_PERFORMANCE_WARNINGS || __TBB_STATISTICS
 #include <typeinfo>
 #endif
+#if __TBB_STATISTICS
+#include <stdio.h>
+#endif
 
 namespace tbb {
 
-template<typename T> struct tbb_hash_compare;
-template<typename Key, typename T, typename HashCompare = tbb_hash_compare<Key>, typename A = tbb_allocator<std::pair<Key, T> > >
-class concurrent_hash_map;
+//! hash_compare that is default argument for concurrent_hash_map
+template<typename Key>
+struct tbb_hash_compare {
+    static size_t hash( const Key& a ) { return tbb_hasher(a); }
+    static bool equal( const Key& a, const Key& b ) { return a == b; }
+};
+
+namespace interface5 {
+
+    template<typename Key, typename T, typename HashCompare = tbb_hash_compare<Key>, typename A = tbb_allocator<std::pair<Key, T> > >
+    class concurrent_hash_map;
+
+    //! @cond INTERNAL
+    namespace internal {
 
-//! @cond INTERNAL
-namespace internal {
-    //! ITT instrumented routine that loads pointer from location pointed to by src.
-    void* __TBB_EXPORTED_FUNC itt_load_pointer_with_acquire_v3( const void* src );
-    //! ITT instrumented routine that stores src into location pointed to by dst.
-    void __TBB_EXPORTED_FUNC itt_store_pointer_with_release_v3( void* dst, void* src );
-    //! Routine that loads pointer from location pointed to by src without causing ITT to report a race.
-    void* __TBB_EXPORTED_FUNC itt_load_pointer_v3( const void* src );
 
     //! Type of a hash code.
     typedef size_t hashcode_t;
     //! Node base type
-    struct hash_map_node_base : no_copy {
+    struct hash_map_node_base : tbb::internal::no_copy {
         //! Mutex type
         typedef spin_rw_mutex mutex_t;
         //! Scoped lock type for mutex
@@ -87,7 +106,7 @@ namespace internal {
         //! Node base type
         typedef hash_map_node_base node_base;
         //! Bucket type
-        struct bucket : no_copy {
+        struct bucket : tbb::internal::no_copy {
             //! Mutex type for buckets
             typedef spin_rw_mutex mutex_t;
             //! Scoped lock type for mutex
@@ -107,7 +126,7 @@ namespace internal {
         typedef bucket *segment_ptr_t;
         //! Segment pointers table type
         typedef segment_ptr_t segments_table_t[pointers_per_table];
-        //! Hash mask = sum of allocated segments sizes - 1
+        //! Hash mask = sum of allocated segment sizes - 1
         atomic<hashcode_t> my_mask;
         //! Segment pointers table. Also prevents false sharing between my_mask and my_size
         segments_table_t my_table;
@@ -115,7 +134,11 @@ namespace internal {
         atomic<size_type> my_size; // It must be in separate cache line from my_mask due to performance effects
         //! Zero segment
         bucket my_embedded_segment[embedded_buckets];
-
+#if __TBB_STATISTICS
+        atomic<unsigned> my_info_resizes; // concurrent ones
+        mutable atomic<unsigned> my_info_restarts; // race collisions
+        atomic<unsigned> my_info_rehashes;  // invocations of rehash_bucket
+#endif
         //! Constructor
         hash_map_base() {
             std::memset( this, 0, pointers_per_table*sizeof(segment_ptr_t) // 32*4=128   or 64*8=512
@@ -125,6 +148,11 @@ namespace internal {
                 my_table[i] = my_embedded_segment + segment_base(i);
             my_mask = embedded_buckets - 1;
             __TBB_ASSERT( embedded_block <= first_block, "The first block number must include embedded blocks");
+#if __TBB_STATISTICS
+            my_info_resizes = 0; // concurrent ones
+            my_info_restarts = 0; // race collisions
+            my_info_rehashes = 0;  // invocations of rehash_bucket
+#endif
         }
 
         //! @return segment index of given index in the array
@@ -183,12 +211,7 @@ namespace internal {
                 sz = segment_size( k );
                 segment_ptr_t ptr = alloc.allocate( sz );
                 init_buckets( ptr, sz, is_initial );
-#if TBB_USE_THREADING_TOOLS
-                // TODO: actually, fence and notification are unnecessary here and below
-                itt_store_pointer_with_release_v3( my_table + k, ptr );
-#else
-                my_table[k] = ptr;// my_mask has release fence
-#endif
+                itt_hide_store_word( my_table[k], ptr );
                 sz <<= 1;// double it to get entire capacity of the container
             } else { // the first block
                 __TBB_ASSERT( k == embedded_block, "Wrong segment index" );
@@ -197,17 +220,9 @@ namespace internal {
                 init_buckets( ptr, sz - embedded_buckets, is_initial );
                 ptr -= segment_base(embedded_block);
                 for(segment_index_t i = embedded_block; i < first_block; i++) // calc the offsets
-#if TBB_USE_THREADING_TOOLS
-                    itt_store_pointer_with_release_v3( my_table + i, ptr + segment_base(i) );
-#else
-                    my_table[i] = ptr + segment_base(i);
-#endif
+                    itt_hide_store_word( my_table[i], ptr + segment_base(i) );
             }
-#if TBB_USE_THREADING_TOOLS
-            itt_store_pointer_with_release_v3( &my_mask, (void*)(sz-1) );
-#else
-            my_mask = sz - 1;
-#endif
+            itt_store_word_with_release( my_mask, sz-1 );
             watchdog.my_segment_ptr = 0;
         }
 
@@ -220,15 +235,21 @@ namespace internal {
             return &seg[h];
         }
 
+        // internal serial rehashing helper
+        void mark_rehashed_levels( hashcode_t h ) throw () {
+            segment_index_t s = segment_index_of( h );
+            while( segment_ptr_t seg = my_table[++s] )
+                if( seg[h].node_list == rehash_req ) {
+                    seg[h].node_list = empty_rehashed;
+                    mark_rehashed_levels( h + ((hashcode_t)1<<s) ); // optimized segment_base(s)
+                }
+        }
+
         //! Check for mask race
         // Splitting into two functions should help inlining
         inline bool check_mask_race( const hashcode_t h, hashcode_t &m ) const {
             hashcode_t m_now, m_old = m;
-#if TBB_USE_THREADING_TOOLS
-            m_now = (hashcode_t) itt_load_pointer_with_acquire_v3( &my_mask );
-#else
-            m_now = my_mask;
-#endif
+            m_now = (hashcode_t) itt_load_word_with_acquire( my_mask );
             if( m_old != m_now )
                 return check_rehashing_collision( h, m_old, m = m_now );
             return false;
@@ -240,16 +261,18 @@ namespace internal {
             if( (h & m_old) != (h & m) ) { // mask changed for this hashcode, rare event
                 // condition above proves that 'h' has some other bits set beside 'm_old'
                 // find next applicable mask after m_old    //TODO: look at bsl instruction
-                for( ++m_old; !(h & m_old); m_old <<= 1 ); // at maximum few rounds depending on the first block size
+                for( ++m_old; !(h & m_old); m_old <<= 1 ) // at maximum few rounds depending on the first block size
+                    ;
                 m_old = (m_old<<1) - 1; // get full mask from a bit
                 __TBB_ASSERT((m_old&(m_old+1))==0 && m_old <= m, NULL);
                 // check whether it is rehashing/ed
-#if TBB_USE_THREADING_TOOLS
-                if( itt_load_pointer_with_acquire_v3(&( get_bucket(h & m_old)->node_list )) != rehash_req )
-#else
-                if( __TBB_load_with_acquire(get_bucket( h & m_old )->node_list) != rehash_req )
+                if( itt_load_word_with_acquire(get_bucket(h & m_old)->node_list) != rehash_req )
+                {
+#if __TBB_STATISTICS
+                    my_info_restarts++; // race collisions
 #endif
                     return true;
+                }
             }
             return false;
         }
@@ -260,13 +283,9 @@ namespace internal {
             add_to_bucket( b, n );
             // check load factor
             if( sz >= mask ) { // TODO: add custom load_factor 
-                segment_index_t new_seg = segment_index_of( mask+1 );
+                segment_index_t new_seg = __TBB_Log2( mask+1 ); //optimized segment_index_of
                 __TBB_ASSERT( is_valid(my_table[new_seg-1]), "new allocations must not publish new mask until segment has allocated");
-#if TBB_USE_THREADING_TOOLS
-                if( !itt_load_pointer_v3(my_table+new_seg)
-#else
-                if( !my_table[new_seg]
-#endif
+                if( !itt_hide_load_word(my_table[new_seg])
                   && __TBB_CompareAndSwapW(&my_table[new_seg], 2, 0) == 0 )
                     return new_seg; // The value must be processed
             }
@@ -316,10 +335,10 @@ namespace internal {
         friend ptrdiff_t operator-( const hash_map_iterator<C,T>& i, const hash_map_iterator<C,U>& j );
     
         template<typename C, typename U>
-        friend class internal::hash_map_iterator;
+        friend class hash_map_iterator;
 
         template<typename I>
-        friend class internal::hash_map_range;
+        friend class hash_map_range;
 
         void advance_to_next_bucket() { // TODO?: refactor to iterator_base class
             size_t k = my_index+1;
@@ -338,7 +357,7 @@ namespace internal {
         }
 #if !defined(_MSC_VER) || defined(__INTEL_COMPILER)
         template<typename Key, typename T, typename HashCompare, typename A>
-        friend class tbb::concurrent_hash_map;
+        friend class interface5::concurrent_hash_map;
 #else
     public: // workaround
 #endif
@@ -373,10 +392,10 @@ namespace internal {
         hash_map_iterator& operator++();
         
         //! Post increment
-        Value* operator++(int) {
-            Value* result = &operator*();
+        hash_map_iterator operator++(int) {
+            hash_map_iterator old(*this);
             operator++();
-            return result;
+            return old;
         }
     };
 
@@ -456,24 +475,24 @@ namespace internal {
         {}
 #if TBB_DEPRECATED
         //! Init range with iterators and grainsize specified
-        hash_map_range( const Iterator& begin_, const Iterator& end_, size_type grainsize = 1 ) : 
+        hash_map_range( const Iterator& begin_, const Iterator& end_, size_type grainsize_ = 1 ) : 
             my_begin(begin_), 
             my_end(end_),
-            my_grainsize(grainsize)
+            my_grainsize(grainsize_)
         {
             if(!my_end.my_index && !my_end.my_bucket) // end
                 my_end.my_index = my_end.my_map->my_mask + 1;
             set_midpoint();
-            __TBB_ASSERT( grainsize>0, "grainsize must be positive" );
+            __TBB_ASSERT( grainsize_>0, "grainsize must be positive" );
         }
 #endif
         //! Init range with container and grainsize specified
-        hash_map_range( const map_type &map, size_type grainsize = 1 ) : 
+        hash_map_range( const map_type &map, size_type grainsize_ = 1 ) : 
             my_begin( Iterator( map, 0, map.my_embedded_segment, map.my_embedded_segment->node_list ) ),
             my_end( Iterator( map, map.my_mask + 1, 0, 0 ) ),
-            my_grainsize( grainsize )
+            my_grainsize( grainsize_ )
         {
-            __TBB_ASSERT( grainsize>0, "grainsize must be positive" );
+            __TBB_ASSERT( grainsize_>0, "grainsize must be positive" );
             set_midpoint();
         }
         const Iterator& begin() const {return my_begin;}
@@ -500,40 +519,10 @@ namespace internal {
         __TBB_ASSERT( my_begin != my_midpoint || my_begin == my_end,
             "[my_begin, my_midpoint) range should not be empty" );
     }
-} // namespace internal
+
+    } // internal
 //! @endcond
 
-//! Hash multiplier
-static const size_t hash_multiplier = sizeof(size_t)==4? 2654435769U : 11400714819323198485ULL;
-//! Hasher functions
-template<typename T>
-inline static size_t tbb_hasher( const T& t ) {
-    return static_cast<size_t>( t ) * hash_multiplier;
-}
-template<typename P>
-inline static size_t tbb_hasher( P* ptr ) {
-    size_t const h = reinterpret_cast<size_t>( ptr );
-    return (h >> 3) ^ h;
-}
-template<typename E, typename S, typename A>
-inline static size_t tbb_hasher( const std::basic_string<E,S,A>& s ) {
-    size_t h = 0;
-    for( const E* c = s.c_str(); *c; c++ )
-        h = static_cast<size_t>(*c) ^ (h * hash_multiplier);
-    return h;
-}
-template<typename F, typename S>
-inline static size_t tbb_hasher( const std::pair<F,S>& p ) {
-    return tbb_hasher(p.first) ^ tbb_hasher(p.second);
-}
-
-//! hash_compare - default argument
-template<typename T>
-struct tbb_hash_compare {
-    static size_t hash( const T& t ) { return tbb_hasher(t); }
-    static bool equal( const T& a, const T& b ) { return a == b; }
-};
-
 //! Unordered map from Key to T.
 /** concurrent_hash_map is associative container with concurrent access.
 
@@ -576,7 +565,7 @@ public:
     typedef Key key_type;
     typedef T mapped_type;
     typedef std::pair<const Key,T> value_type;
-    typedef internal::hash_map_base::size_type size_type;
+    typedef hash_map_base::size_type size_type;
     typedef ptrdiff_t difference_type;
     typedef value_type *pointer;
     typedef const value_type *const_pointer;
@@ -602,7 +591,8 @@ protected:
         // exception-safe allocation, see C++ Standard 2003, clause 5.3.4p17
         void *operator new( size_t /*size*/, node_allocator_type &a ) {
             void *ptr = a.allocate(1);
-            if(!ptr) throw std::bad_alloc();
+            if(!ptr) 
+                tbb::internal::throw_exception(tbb::internal::eid_bad_alloc);
             return ptr;
         }
         // match placement-new form above to be called if exception thrown in constructor
@@ -624,33 +614,25 @@ protected:
 
     //! bucket accessor is to find, rehash, acquire a lock, and access a bucket
     class bucket_accessor : public bucket::scoped_t {
-        bool my_is_writer; // TODO: use it from base type
         bucket *my_b;
     public:
         bucket_accessor( concurrent_hash_map *base, const hashcode_t h, bool writer = false ) { acquire( base, h, writer ); }
         //! find a bucket by masked hashcode, optionally rehash, and acquire the lock
         inline void acquire( concurrent_hash_map *base, const hashcode_t h, bool writer = false ) {
             my_b = base->get_bucket( h );
-#if TBB_USE_THREADING_TOOLS
             // TODO: actually, notification is unnecessary here, just hiding double-check
-            if( itt_load_pointer_with_acquire_v3(&my_b->node_list) == internal::rehash_req
-#else
-            if( __TBB_load_with_acquire(my_b->node_list) == internal::rehash_req
-#endif
+            if( itt_load_word_with_acquire(my_b->node_list) == internal::rehash_req
                 && try_acquire( my_b->mutex, /*write=*/true ) )
             {
                 if( my_b->node_list == internal::rehash_req ) base->rehash_bucket( my_b, h ); //recursive rehashing
-                my_is_writer = true;
             }
-            else bucket::scoped_t::acquire( my_b->mutex, /*write=*/my_is_writer = writer );
+            else bucket::scoped_t::acquire( my_b->mutex, writer );
             __TBB_ASSERT( my_b->node_list != internal::rehash_req, NULL);
         }
         //! check whether bucket is locked for write
-        bool is_writer() { return my_is_writer; }
+        bool is_writer() { return bucket::scoped_t::is_writer; }
         //! get bucket pointer
         bucket *operator() () { return my_b; }
-        // TODO: optimize out
-        bool upgrade_to_writer() { my_is_writer = true; return bucket::scoped_t::upgrade_to_writer(); }
     };
 
     // TODO refactor to hash_base
@@ -659,6 +641,9 @@ protected:
         __TBB_ASSERT( h > 1, "The lowermost buckets can't be rehashed" );
         __TBB_store_with_release(b_new->node_list, internal::empty_rehashed); // mark rehashed
         hashcode_t mask = ( 1u<<__TBB_Log2( h ) ) - 1; // get parent mask from the topmost bit
+#if __TBB_STATISTICS
+        my_info_rehashes++; // invocations of rehash_bucket
+#endif
 
         bucket_accessor b_old( this, h & mask );
 
@@ -667,6 +652,11 @@ protected:
     restart:
         for( node_base **p = &b_old()->node_list, *n = __TBB_load_with_acquire(*p); is_valid(n); n = *p ) {
             hashcode_t c = my_hash_compare.hash( static_cast<node*>(n)->item.first );
+#if TBB_USE_ASSERT
+            hashcode_t bmask = h & (mask>>1);
+            bmask = bmask==0? 1 : ( 1u<<(__TBB_Log2( bmask )+1 ) ) - 1; // minimal mask of parent bucket
+            __TBB_ASSERT( (c & bmask) == (h & bmask), "hash() function changed for key in table" );
+#endif
             if( (c & mask) == h ) {
                 if( !b_old.is_writer() )
                     if( !b_old.upgrade_to_writer() ) {
@@ -682,11 +672,9 @@ public:
     
     class accessor;
     //! Combines data access, locking, and garbage collection.
-    class const_accessor {
+    class const_accessor : private node::scoped_t /*which derived from no_copy*/ {
         friend class concurrent_hash_map<Key,T,HashCompare,Allocator>;
         friend class accessor;
-        void operator=( const accessor & ) const; // Deny access
-        const_accessor( const accessor & );       // Deny access
     public:
         //! Type of value
         typedef const typename concurrent_hash_map::value_type value_type;
@@ -697,7 +685,7 @@ public:
         //! Set to null
         void release() {
             if( my_node ) {
-                my_lock.release();
+                node::scoped_t::release();
                 my_node = 0;
             }
         }
@@ -718,11 +706,11 @@ public:
 
         //! Destroy result after releasing the underlying reference.
         ~const_accessor() {
-            my_node = NULL; // my_lock.release() is called in scoped_lock destructor
+            my_node = NULL; // scoped lock's release() is called in its destructor
         }
-    private:
+    protected:
+        bool is_writer() { return node::scoped_t::is_writer; }
         node *my_node;
-        typename node::scoped_t my_lock;
         hashcode_t my_hash;
     };
 
@@ -746,7 +734,7 @@ public:
 
     //! Construct empty table.
     concurrent_hash_map(const allocator_type &a = allocator_type())
-        : my_allocator(a)
+        : internal::hash_map_base(), my_allocator(a)
     {}
 
     //! Construct empty table with n preallocated buckets. This number serves also as initial concurrency level.
@@ -758,7 +746,7 @@ public:
 
     //! Copy constructor
     concurrent_hash_map( const concurrent_hash_map& table, const allocator_type &a = allocator_type())
-        : my_allocator(a)
+        : internal::hash_map_base(), my_allocator(a)
     {
         internal_copy(table);
     }
@@ -782,6 +770,11 @@ public:
     }
 
 
+    //! Rehashes and optionally resizes the whole table.
+    /** Useful to optimize performance before or after concurrent operations.
+        Also enables using of find() and count() concurrent methods in serial context. */
+    void rehash(size_type n = 0);
+    
     //! Clear table
     void clear();
 
@@ -817,6 +810,9 @@ public:
     //! Upper bound on size.
     size_type max_size() const {return (~size_type(0))/sizeof(node);}
 
+    //! Returns the current number of buckets
+    size_type bucket_count() const { return my_mask+1; }
+
     //! return allocator object
     allocator_type get_allocator() const { return this->my_allocator; }
 
@@ -894,13 +890,13 @@ public:
     //! Erase item by const_accessor.
     /** Return true if item was erased by particularly this call. */
     bool erase( const_accessor& item_accessor ) {
-        return exclude( item_accessor, /*readonly=*/ true );
+        return exclude( item_accessor );
     }
 
     //! Erase item by accessor.
     /** Return true if item was erased by particularly this call. */
     bool erase( accessor& item_accessor ) {
-        return exclude( item_accessor, /*readonly=*/ false );
+        return exclude( item_accessor );
     }
 
 protected:
@@ -908,7 +904,7 @@ protected:
     bool lookup( bool op_insert, const Key &key, const T *t, const_accessor *result, bool write );
 
     //! delete item by accessor
-    bool exclude( const_accessor &item_accessor, bool readonly );
+    bool exclude( const_accessor &item_accessor );
 
     //! Returns an iterator for an item defined by the key, or for the next item after it (if upper==true)
     template<typename I>
@@ -925,21 +921,13 @@ protected:
         Must not be called concurrently with erasure operations. */
     const_pointer internal_fast_find( const Key& key ) const {
         hashcode_t h = my_hash_compare.hash( key );
-#if TBB_USE_THREADING_TOOLS
-        hashcode_t m = (hashcode_t) itt_load_pointer_with_acquire_v3( &my_mask );
-#else
-        hashcode_t m = my_mask;
-#endif
+        hashcode_t m = (hashcode_t) itt_load_word_with_acquire( my_mask );
         node *n;
     restart:
         __TBB_ASSERT((m&(m+1))==0, NULL);
         bucket *b = get_bucket( h & m );
-#if TBB_USE_THREADING_TOOLS
         // TODO: actually, notification is unnecessary here, just hiding double-check
-        if( itt_load_pointer_with_acquire_v3(&b->node_list) == internal::rehash_req )
-#else
-        if( __TBB_load_with_acquire(b->node_list) == internal::rehash_req )
-#endif
+        if( itt_load_word_with_acquire(b->node_list) == internal::rehash_req )
         {
             bucket::scoped_t lock;
             if( lock.try_acquire( b->mutex, /*write=*/true ) ) {
@@ -967,15 +955,11 @@ protected:
 template<typename Key, typename T, typename HashCompare, typename A>
 bool concurrent_hash_map<Key,T,HashCompare,A>::lookup( bool op_insert, const Key &key, const T *t, const_accessor *result, bool write ) {
     __TBB_ASSERT( !result || !result->my_node, NULL );
-    segment_index_t grow_segment;
     bool return_value;
-    node *n, *tmp_n = 0;
     hashcode_t const h = my_hash_compare.hash( key );
-#if TBB_USE_THREADING_TOOLS
-    hashcode_t m = (hashcode_t) itt_load_pointer_with_acquire_v3( &my_mask );
-#else
-    hashcode_t m = my_mask;
-#endif
+    hashcode_t m = (hashcode_t) itt_load_word_with_acquire( my_mask );
+    segment_index_t grow_segment = 0;
+    node *n, *tmp_n = 0;
     restart:
     {//lock scope
         __TBB_ASSERT((m&(m+1))==0, NULL);
@@ -1006,8 +990,6 @@ bool concurrent_hash_map<Key,T,HashCompare,A>::lookup( bool op_insert, const Key
                 grow_segment = insert_new_node( b(), n = tmp_n, m );
                 tmp_n = 0;
                 return_value = true;
-            } else {
-    exists:     grow_segment = 0;
             }
         } else { // find or count
             if( !n ) {
@@ -1016,32 +998,36 @@ bool concurrent_hash_map<Key,T,HashCompare,A>::lookup( bool op_insert, const Key
                 return false;
             }
             return_value = true;
-            grow_segment = 0;
         }
+    exists:
         if( !result ) goto check_growth;
         // TODO: the following seems as generic/regular operation
         // acquire the item
-        if( !result->my_lock.try_acquire( n->mutex, write ) ) {
+        if( !result->try_acquire( n->mutex, write ) ) {
             // we are unlucky, prepare for longer wait
-            internal::atomic_backoff trials;
+            tbb::internal::atomic_backoff trials;
             do {
                 if( !trials.bounded_pause() ) {
                     // the wait takes really long, restart the operation
                     b.release();
                     __TBB_ASSERT( !op_insert || !return_value, "Can't acquire new item in locked bucket?" );
                     __TBB_Yield();
-                    m = my_mask;
+                    m = (hashcode_t) itt_load_word_with_acquire( my_mask );
                     goto restart;
                 }
-            } while( !result->my_lock.try_acquire( n->mutex, write ) );
+            } while( !result->try_acquire( n->mutex, write ) );
         }
     }//lock scope
     result->my_node = n;
     result->my_hash = h;
 check_growth:
     // [opt] grow the container
-    if( grow_segment )
+    if( grow_segment ) {
+#if __TBB_STATISTICS
+        my_info_resizes++; // concurrent ones
+#endif
         enable_segment( grow_segment );
+    }
     if( tmp_n ) // if op_insert only
         delete_node( tmp_n );
     return return_value;
@@ -1049,7 +1035,7 @@ check_growth:
 
 template<typename Key, typename T, typename HashCompare, typename A>
 template<typename I>
-std::pair<I, I> concurrent_hash_map<Key,T,HashCompare,A>::internal_equal_range( const Key& key, I end ) const {
+std::pair<I, I> concurrent_hash_map<Key,T,HashCompare,A>::internal_equal_range( const Key& key, I end_ ) const {
     hashcode_t h = my_hash_compare.hash( key );
     hashcode_t m = my_mask;
     __TBB_ASSERT((m&(m+1))==0, NULL);
@@ -1061,18 +1047,17 @@ std::pair<I, I> concurrent_hash_map<Key,T,HashCompare,A>::internal_equal_range(
     }
     node *n = search_bucket( key, b );
     if( !n )
-        return std::make_pair(end, end);
+        return std::make_pair(end_, end_);
     iterator lower(*this, h, b, n), upper(lower);
     return std::make_pair(lower, ++upper);
 }
 
 template<typename Key, typename T, typename HashCompare, typename A>
-bool concurrent_hash_map<Key,T,HashCompare,A>::exclude( const_accessor &item_accessor, bool readonly ) {
+bool concurrent_hash_map<Key,T,HashCompare,A>::exclude( const_accessor &item_accessor ) {
     __TBB_ASSERT( item_accessor.my_node, NULL );
     node_base *const n = item_accessor.my_node;
-    item_accessor.my_node = NULL; // we ought release accessor anyway
     hashcode_t const h = item_accessor.my_hash;
-    hashcode_t m = my_mask;
+    hashcode_t m = (hashcode_t) itt_load_word_with_acquire( my_mask );
     do {
         // get bucket
         bucket_accessor b( this, h & m, /*writer=*/true );
@@ -1082,7 +1067,7 @@ bool concurrent_hash_map<Key,T,HashCompare,A>::exclude( const_accessor &item_acc
         if( !*p ) { // someone else was the first
             if( check_mask_race( h, m ) )
                 continue;
-            item_accessor.my_lock.release();
+            item_accessor.release();
             return false;
         }
         __TBB_ASSERT( *p == n, NULL );
@@ -1090,10 +1075,10 @@ bool concurrent_hash_map<Key,T,HashCompare,A>::exclude( const_accessor &item_acc
         my_size--;
         break;
     } while(true);
-    if( readonly ) // need to get exclusive lock
-        item_accessor.my_lock.upgrade_to_writer(); // return value means nothing here
-    item_accessor.my_lock.release();
-    delete_node( n ); // Only one thread can delete it due to write lock on the chain_mutex
+    if( !item_accessor.is_writer() ) // need to get exclusive lock
+        item_accessor.upgrade_to_writer(); // return value means nothing here
+    item_accessor.release();
+    delete_node( n ); // Only one thread can delete it
     return true;
 }
 
@@ -1101,7 +1086,7 @@ template<typename Key, typename T, typename HashCompare, typename A>
 bool concurrent_hash_map<Key,T,HashCompare,A>::erase( const Key &key ) {
     node_base *n;
     hashcode_t const h = my_hash_compare.hash( key );
-    hashcode_t m = my_mask;
+    hashcode_t m = (hashcode_t) itt_load_word_with_acquire( my_mask );
 restart:
     {//lock scope
         // get bucket
@@ -1141,57 +1126,139 @@ void concurrent_hash_map<Key,T,HashCompare,A>::swap(concurrent_hash_map<Key,T,Ha
     internal_swap(table);
 }
 
+template<typename Key, typename T, typename HashCompare, typename A>
+void concurrent_hash_map<Key,T,HashCompare,A>::rehash(size_type sz) {
+    reserve( sz ); // TODO: add reduction of number of buckets as well
+    hashcode_t mask = my_mask;
+    hashcode_t b = (mask+1)>>1; // size or first index of the last segment
+    __TBB_ASSERT((b&(b-1))==0, NULL);
+    bucket *bp = get_bucket( b ); // only the last segment should be scanned for rehashing
+    for(; b <= mask; b++, bp++ ) {
+        node_base *n = bp->node_list;
+        __TBB_ASSERT( is_valid(n) || n == internal::empty_rehashed || n == internal::rehash_req, "Broken internal structure" );
+        __TBB_ASSERT( *reinterpret_cast<intptr_t*>(&bp->mutex) == 0, "concurrent or unexpectedly terminated operation during rehash() execution" );
+        if( n == internal::rehash_req ) { // rehash bucket, conditional because rehashing of a previous bucket may affect this one
+            hashcode_t h = b; bucket *b_old = bp;
+            do {
+                __TBB_ASSERT( h > 1, "The lowermost buckets can't be rehashed" );
+                hashcode_t m = ( 1u<<__TBB_Log2( h ) ) - 1; // get parent mask from the topmost bit
+                b_old = get_bucket( h &= m );
+            } while( b_old->node_list == internal::rehash_req );
+            // now h - is index of the root rehashed bucket b_old
+            mark_rehashed_levels( h ); // mark all non-rehashed children recursively across all segments
+            for( node_base **p = &b_old->node_list, *q = *p; is_valid(q); q = *p ) {
+                hashcode_t c = my_hash_compare.hash( static_cast<node*>(q)->item.first );
+                if( (c & mask) != h ) { // should be rehashed
+                    *p = q->next; // exclude from b_old
+                    bucket *b_new = get_bucket( c & mask );
+                    __TBB_ASSERT( b_new->node_list != internal::rehash_req, "hash() function changed for key in table or internal error" );
+                    add_to_bucket( b_new, q );
+                } else p = &q->next; // iterate to next item
+            }
+        }
+    }
+#if TBB_USE_PERFORMANCE_WARNINGS
+    int current_size = int(my_size), buckets = int(mask)+1, empty_buckets = 0, overpopulated_buckets = 0; // usage statistics
+    static bool reported = false;
+#endif
+#if TBB_USE_ASSERT || TBB_USE_PERFORMANCE_WARNINGS
+    for( b = 0; b <= mask; b++ ) {// only last segment should be scanned for rehashing
+        if( b & (b-2) ) ++bp; // not the beginning of a segment
+        else bp = get_bucket( b );
+        node_base *n = bp->node_list;
+        __TBB_ASSERT( *reinterpret_cast<intptr_t*>(&bp->mutex) == 0, "concurrent or unexpectedly terminated operation during rehash() execution" );
+        __TBB_ASSERT( is_valid(n) || n == internal::empty_rehashed, "Broken internal structure" );
+#if TBB_USE_PERFORMANCE_WARNINGS
+        if( n == internal::empty_rehashed ) empty_buckets++;
+        else if( n->next ) overpopulated_buckets++;
+#endif
+#if TBB_USE_ASSERT
+        for( ; is_valid(n); n = n->next ) {
+            hashcode_t h = my_hash_compare.hash( static_cast<node*>(n)->item.first ) & mask;
+            __TBB_ASSERT( h == b, "hash() function changed for key in table or internal error" );
+        }
+#endif
+    }
+#endif // TBB_USE_ASSERT || TBB_USE_PERFORMANCE_WARNINGS
+#if TBB_USE_PERFORMANCE_WARNINGS
+    if( buckets > current_size) empty_buckets -= buckets - current_size;
+    else overpopulated_buckets -= current_size - buckets; // TODO: load_factor?
+    if( !reported && buckets >= 512 && ( 2*empty_buckets > current_size || 2*overpopulated_buckets > current_size ) ) {
+        tbb::internal::runtime_warning(
+            "Performance is not optimal because the hash function produces bad randomness in lower bits in %s.\nSize: %d  Empties: %d  Overlaps: %d",
+            typeid(*this).name(), current_size, empty_buckets, overpopulated_buckets );
+        reported = true;
+    }
+#endif
+}
+
 template<typename Key, typename T, typename HashCompare, typename A>
 void concurrent_hash_map<Key,T,HashCompare,A>::clear() {
     hashcode_t m = my_mask;
     __TBB_ASSERT((m&(m+1))==0, NULL);
-#if TBB_USE_DEBUG || TBB_USE_PERFORMANCE_WARNINGS
-#if TBB_USE_PERFORMANCE_WARNINGS
-    int size = int(my_size), buckets = int(m)+1, empty_buckets = 0, overpopulated_buckets = 0; // usage statistics
+#if TBB_USE_ASSERT || TBB_USE_PERFORMANCE_WARNINGS || __TBB_STATISTICS
+#if TBB_USE_PERFORMANCE_WARNINGS || __TBB_STATISTICS
+    int current_size = int(my_size), buckets = int(m)+1, empty_buckets = 0, overpopulated_buckets = 0; // usage statistics
     static bool reported = false;
 #endif
+    bucket *bp = 0;
     // check consistency
     for( segment_index_t b = 0; b <= m; b++ ) {
-        node_base *n = get_bucket(b)->node_list;
-#if TBB_USE_PERFORMANCE_WARNINGS
+        if( b & (b-2) ) ++bp; // not the beginning of a segment
+        else bp = get_bucket( b );
+        node_base *n = bp->node_list;
+        __TBB_ASSERT( is_valid(n) || n == internal::empty_rehashed || n == internal::rehash_req, "Broken internal structure" );
+        __TBB_ASSERT( *reinterpret_cast<intptr_t*>(&bp->mutex) == 0, "concurrent or unexpectedly terminated operation during clear() execution" );
+#if TBB_USE_PERFORMANCE_WARNINGS || __TBB_STATISTICS
         if( n == internal::empty_rehashed ) empty_buckets++;
         else if( n == internal::rehash_req ) buckets--;
         else if( n->next ) overpopulated_buckets++;
 #endif
+#if __TBB_EXTRA_DEBUG
         for(; is_valid(n); n = n->next ) {
             hashcode_t h = my_hash_compare.hash( static_cast<node*>(n)->item.first );
             h &= m;
-            __TBB_ASSERT( h == b || get_bucket(h)->node_list == internal::rehash_req, "Rehashing is not finished until serial stage due to concurrent or unexpectedly terminated operation" );
+            __TBB_ASSERT( h == b || get_bucket(h)->node_list == internal::rehash_req, "hash() function changed for key in table or internal error" );
         }
+#endif
     }
-#if TBB_USE_PERFORMANCE_WARNINGS
-    if( buckets > size) empty_buckets -= buckets - size;
-    else overpopulated_buckets -= size - buckets; // TODO: load_factor?
-    if( !reported && buckets >= 512 && ( 2*empty_buckets >= size || 2*overpopulated_buckets > size ) ) {
-        internal::runtime_warning(
+#if TBB_USE_PERFORMANCE_WARNINGS || __TBB_STATISTICS
+#if __TBB_STATISTICS
+    printf( "items=%d buckets: capacity=%d rehashed=%d empty=%d overpopulated=%d"
+        " concurrent: resizes=%u rehashes=%u restarts=%u\n",
+        current_size, int(m+1), buckets, empty_buckets, overpopulated_buckets,
+        unsigned(my_info_resizes), unsigned(my_info_rehashes), unsigned(my_info_restarts) );
+    my_info_resizes = 0; // concurrent ones
+    my_info_restarts = 0; // race collisions
+    my_info_rehashes = 0;  // invocations of rehash_bucket
+#endif
+    if( buckets > current_size) empty_buckets -= buckets - current_size;
+    else overpopulated_buckets -= current_size - buckets; // TODO: load_factor?
+    if( !reported && buckets >= 512 && ( 2*empty_buckets > current_size || 2*overpopulated_buckets > current_size ) ) {
+        tbb::internal::runtime_warning(
             "Performance is not optimal because the hash function produces bad randomness in lower bits in %s.\nSize: %d  Empties: %d  Overlaps: %d",
-            typeid(*this).name(), size, empty_buckets, overpopulated_buckets );
+            typeid(*this).name(), current_size, empty_buckets, overpopulated_buckets );
         reported = true;
     }
 #endif
-#endif//TBB_USE_DEBUG || TBB_USE_PERFORMANCE_WARNINGS
+#endif//TBB_USE_ASSERT || TBB_USE_PERFORMANCE_WARNINGS || __TBB_STATISTICS
     my_size = 0;
     segment_index_t s = segment_index_of( m );
     __TBB_ASSERT( s+1 == pointers_per_table || !my_table[s+1], "wrong mask or concurrent grow" );
     cache_aligned_allocator<bucket> alloc;
     do {
         __TBB_ASSERT( is_valid( my_table[s] ), "wrong mask or concurrent grow" );
-        segment_ptr_t buckets = my_table[s];
+        segment_ptr_t buckets_ptr = my_table[s];
         size_type sz = segment_size( s ? s : 1 );
         for( segment_index_t i = 0; i < sz; i++ )
-            for( node_base *n = buckets[i].node_list; is_valid(n); n = buckets[i].node_list ) {
-                buckets[i].node_list = n->next;
+            for( node_base *n = buckets_ptr[i].node_list; is_valid(n); n = buckets_ptr[i].node_list ) {
+                buckets_ptr[i].node_list = n->next;
                 delete_node( n );
             }
         if( s >= first_block) // the first segment or the next
-            alloc.deallocate( buckets, sz );
+            alloc.deallocate( buckets_ptr, sz );
         else if( s == embedded_block && embedded_block != first_block )
-            alloc.deallocate( buckets, segment_size(first_block)-embedded_buckets );
+            alloc.deallocate( buckets_ptr, segment_size(first_block)-embedded_buckets );
         if( s >= embedded_block ) my_table[s] = 0;
     } while(s-- > 0);
     my_mask = embedded_buckets - 1;
@@ -1203,19 +1270,21 @@ void concurrent_hash_map<Key,T,HashCompare,A>::internal_copy( const concurrent_h
     hashcode_t mask = source.my_mask;
     if( my_mask == mask ) { // optimized version
         bucket *dst = 0, *src = 0;
+        bool rehash_required = false;
         for( hashcode_t k = 0; k <= mask; k++ ) {
             if( k & (k-2) ) ++dst,src++; // not the beginning of a segment
             else { dst = get_bucket( k ); src = source.get_bucket( k ); }
             __TBB_ASSERT( dst->node_list != internal::rehash_req, "Invalid bucket in destination table");
             node *n = static_cast<node*>( src->node_list );
             if( n == internal::rehash_req ) { // source is not rehashed, items are in previous buckets
-                bucket_accessor b( this, k );
-                rehash_bucket( b(), k ); // TODO: use without synchronization
+                rehash_required = true;
+                dst->node_list = internal::rehash_req;
             } else for(; n; n = static_cast<node*>( n->next ) ) {
                 add_to_bucket( dst, new( my_allocator ) node(n->item.first, n->item.second) );
                 ++my_size; // TODO: replace by non-atomic op
             }
         }
+        if( rehash_required ) rehash();
     } else internal_copy( source.begin(), source.end() );
 }
 
@@ -1233,6 +1302,11 @@ void concurrent_hash_map<Key,T,HashCompare,A>::internal_copy(I first, I last) {
     }
 }
 
+} // namespace interface5
+
+using interface5::concurrent_hash_map;
+
+
 template<typename Key, typename T, typename HashCompare, typename A1, typename A2>
 inline bool operator==(const concurrent_hash_map<Key, T, HashCompare, A1> &a, const concurrent_hash_map<Key, T, HashCompare, A2> &b) {
     if(a.size() != b.size()) return false;
