@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -30,19 +30,14 @@
 #define __TBB_mutex_H
 
 #if _WIN32||_WIN64
-#include <windows.h>
-#if !defined(_WIN32_WINNT)
-// The following Windows API function is declared explicitly;
-// otherwise any user would have to specify /D_WIN32_WINNT=0x0400
-extern "C" BOOL WINAPI TryEnterCriticalSection( LPCRITICAL_SECTION );
-#endif
-
+    #include <windows.h>
+    #if !defined(_WIN32_WINNT)
+    // The following Windows API function is declared explicitly;
+    // otherwise any user would have to specify /D_WIN32_WINNT=0x0400
+    extern "C" BOOL WINAPI TryEnterCriticalSection( LPCRITICAL_SECTION );
+    #endif
 #else /* if not _WIN32||_WIN64 */
-#include <pthread.h>
-namespace tbb { namespace internal {
-// Use this internal TBB function to throw an exception
-extern void handle_perror( int error_code, const char* what );
-} } //namespaces
+    #include <pthread.h>
 #endif /* _WIN32||_WIN64 */
 
 #include <new>
@@ -97,7 +92,6 @@ public:
         scoped_lock() : my_mutex(NULL) {};
 
         //! Acquire lock on given mutex.
-        /** Upon entry, *this should not be in the "have acquired a mutex" state. */
         scoped_lock( mutex& mutex ) {
             acquire( mutex );
         }
@@ -210,14 +204,23 @@ public:
 #endif /* TBB_USE_ASSERT */
     }
 
-private:
-#if _WIN32||_WIN64
-    CRITICAL_SECTION impl;    
+    //! Return native_handle
+  #if _WIN32||_WIN64
+    typedef LPCRITICAL_SECTION native_handle_type;
+  #else
+    typedef pthread_mutex_t* native_handle_type;
+  #endif
+    native_handle_type native_handle() { return (native_handle_type) &impl; }
+
     enum state_t {
         INITIALIZED=0x1234,
         DESTROYED=0x789A,
         HELD=0x56CD
-    } state;
+    };
+private:
+#if _WIN32||_WIN64
+    CRITICAL_SECTION impl;    
+    enum state_t state;
 #else
     pthread_mutex_t impl;
 #endif /* _WIN32||_WIN64 */
@@ -227,6 +230,12 @@ private:
 
     //! All checks from mutex destructor using mutex.state were moved here
     void __TBB_EXPORTED_METHOD internal_destroy();
+
+#if _WIN32||_WIN64
+public:
+    //!  Set the internal state
+    void set_state( state_t to ) { state = to; }
+#endif
 };
 
 __TBB_DEFINE_PROFILING_SET_NAME(mutex)
