@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -151,7 +151,7 @@ public:
         size_type absolutemax = static_cast<size_type>(-1) / sizeof (value_type);
         return (absolutemax > 0 ? absolutemax : 1);
     }
-    void construct( pointer p, const value_type& val ) { new(static_cast<void*>(p)) value_type(val); }
+    void construct( pointer p, const value_type& value ) {::new((void*)(p)) value_type(value);}
     void destroy( pointer p ) {p->~value_type();}
 };
 
