@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -29,7 +29,7 @@
 #ifndef __TBB_machine_H
 #define __TBB_machine_H
 
-#include "tbb/tbb_stddef.h"
+#include "tbb_stddef.h"
 
 #if _WIN32||_WIN64
 
@@ -37,10 +37,16 @@
 #pragma managed(push, off)
 #endif
 
-#if defined(_M_IX86)
-#include "tbb/machine/windows_ia32.h"
+#if __MINGW32__
+#include "machine/linux_ia32.h"
+extern "C" __declspec(dllimport) int __stdcall SwitchToThread( void );
+#define __TBB_Yield()  SwitchToThread()
+#elif defined(_M_IX86)
+#include "machine/windows_ia32.h"
 #elif defined(_M_AMD64) 
-#include "tbb/machine/windows_em64t.h"
+#include "machine/windows_intel64.h"
+#elif _XBOX 
+#include "machine/xbox360_ppc.h"
 #else
 #error Unsupported platform
 #endif
@@ -52,40 +58,59 @@
 #elif __linux__ || __FreeBSD__
 
 #if __i386__
-#include "tbb/machine/linux_ia32.h"
+#include "machine/linux_ia32.h"
 #elif __x86_64__
-#include "tbb/machine/linux_em64t.h"
+#include "machine/linux_intel64.h"
 #elif __ia64__
-#include "tbb/machine/linux_itanium.h"
+#include "machine/linux_ia64.h"
 #endif
 
 #elif __APPLE__
 
 #if __i386__
-#include "tbb/machine/linux_ia32.h"
+#include "machine/linux_ia32.h"
 #elif __x86_64__
-#include "tbb/machine/linux_em64t.h"
+#include "machine/linux_intel64.h"
 #elif __POWERPC__
-#include "tbb/machine/mac_ppc.h"
+#include "machine/mac_ppc.h"
 #endif
 
 #elif _AIX
 
-#include "tbb/machine/ibm_aix51.h"
+#include "machine/ibm_aix51.h"
 
 #elif __sun || __SUNPRO_CC
 
 #define __asm__ asm 
 #define __volatile__ volatile
 #if __i386  || __i386__
-#include "tbb/machine/linux_ia32.h"
+#include "machine/linux_ia32.h"
 #elif __x86_64__
-#include "tbb/machine/linux_em64t.h"
+#include "machine/linux_intel64.h"
+#elif __sparc
+#include "machine/sunos_sparc.h"
 #endif
 
 #endif
 
-#if !defined(__TBB_CompareAndSwap4) || !defined(__TBB_CompareAndSwap8) || !defined(__TBB_Yield)
+//! Prerequisites for each architecture port
+/** There are no generic implementation for these macros so they have to be implemented
+    in each machine architecture specific header.
+
+    __TBB_full_memory_fence must prevent all memory operations from being reordered 
+    across the fence. And all such fences must be totally ordered (or sequentially 
+    consistent). These fence must affect both compiler and hardware.
+    
+    __TBB_release_consistency_helper is used to enforce guarantees of acquire or 
+    release semantics in generic implementations of __TBB_load_with_acquire and 
+    __TBB_store_with_release below. Depending on the particular combination of
+    architecture+compiler it can be a hardware fence, a compiler fence, both or
+    nothing. **/
+#if    !defined(__TBB_CompareAndSwap4) \
+    || !defined(__TBB_CompareAndSwap8) \
+    || !defined(__TBB_Yield)           \
+    || !defined(__TBB_full_memory_fence)    \
+    || !defined(__TBB_release_consistency_helper)
 #error Minimal requirements for tbb_machine.h not satisfied 
 #endif
 
@@ -94,9 +119,7 @@
     template<typename T>
     inline T __TBB_load_with_acquire(const volatile T& location) {
         T temp = location;
-#ifdef __TBB_fence_for_acquire 
-        __TBB_fence_for_acquire();
-#endif /* __TBB_fence_for_acquire */
+        __TBB_release_consistency_helper();
         return temp;
     }
 #endif
@@ -105,10 +128,8 @@
     //! Store with release semantics; i.e., no prior memory operation can move below the store.
     template<typename T, typename V>
     inline void __TBB_store_with_release(volatile T& location, V value) {
-#ifdef __TBB_fence_for_release
-        __TBB_fence_for_release();
-#endif /* __TBB_fence_for_release */
-        location = value; 
+        __TBB_release_consistency_helper();
+        location = T(value); 
     }
 #endif
 
@@ -122,15 +143,15 @@ namespace tbb {
 namespace internal {
 
 //! Class that implements exponential backoff.
-/** See implementation of SpinwaitWhileEq for an example. */
-class AtomicBackoff {
+/** See implementation of spin_wait_while_eq for an example. */
+class atomic_backoff {
     //! Time delay, in units of "pause" instructions. 
     /** Should be equal to approximately the number of "pause" instructions
         that take the same time as an context switch. */
     static const int32_t LOOPS_BEFORE_YIELD = 16;
     int32_t count;
 public:
-    AtomicBackoff() : count(1) {}
+    atomic_backoff() : count(1) {}
 
     //! Pause for a while.
     void pause() {
@@ -161,28 +182,49 @@ public:
     }
 };
 
+//! Spin WHILE the value of the variable is equal to a given value
+/** T and U should be comparable types. */
+template<typename T, typename U>
+void spin_wait_while_eq( const volatile T& location, U value ) {
+    atomic_backoff backoff;
+    while( location==value ) backoff.pause();
+}
+
+//! Spin UNTIL the value of the variable is equal to a given value
+/** T and U should be comparable types. */
+template<typename T, typename U>
+void spin_wait_until_eq( const volatile T& location, const U value ) {
+    atomic_backoff backoff;
+    while( location!=value ) backoff.pause();
+}
+
+// T should be unsigned, otherwise sign propagation will break correctness of bit manipulations.
+// S should be either 1 or 2, for the mask calculation to work correctly.
+// Together, these rules limit applicability of Masked CAS to unsigned char and unsigned short.
 template<size_t S, typename T>
-inline intptr_t __TBB_MaskedCompareAndSwap (volatile int32_t *ptr, T value, T comparand ) {
-    T *base = (T *)( (uintptr_t)(ptr) & ~(uintptr_t)(0x3) );
+inline T __TBB_MaskedCompareAndSwap (volatile T *ptr, T value, T comparand ) {
+    volatile uint32_t * base = (uint32_t*)( (uintptr_t)ptr & ~(uintptr_t)0x3 );
 #if __TBB_BIG_ENDIAN
-    const uint8_t bitoffset = ( (4-S) - ( (uint8_t *)ptr - (uint8_t *)base) ) * 8;
+    const uint8_t bitoffset = uint8_t( 8*( 4-S - (uintptr_t(ptr) & 0x3) ) );
 #else
-    const uint8_t bitoffset = ( (uint8_t *)ptr - (uint8_t *)base ) * 8;
+    const uint8_t bitoffset = uint8_t( 8*((uintptr_t)ptr & 0x3) );
 #endif
-    const uint32_t mask = ( (1<<(S*8) ) - 1)<<bitoffset;
-    AtomicBackoff b;
+    const uint32_t mask = ( (1<<(S*8)) - 1 )<<bitoffset;
+    atomic_backoff b;
     uint32_t result;
     for(;;) {
-        result = *(volatile uint32_t *)base;
+        result = *base; // reload the base value which might change during the pause
         uint32_t old_value = ( result & ~mask ) | ( comparand << bitoffset );
         uint32_t new_value = ( result & ~mask ) | ( value << bitoffset );
         // __TBB_CompareAndSwap4 presumed to have full fence. 
-        uint32_t tmp = __TBB_CompareAndSwap4( base, new_value, old_value );
-        if( tmp==old_value || ((tmp^old_value)&mask)!=0 ) 
+        result = __TBB_CompareAndSwap4( base, new_value, old_value );
+        if(  result==old_value               // CAS succeeded
+          || ((result^old_value)&mask)!=0 )  // CAS failed and the bits of interest have changed
             break;
-        b.pause();
+        else                                 // CAS failed but the bits of interest left unchanged
+            b.pause();
     }
-    return (T)((result & mask) >> bitoffset);
+    return T((result & mask) >> bitoffset);
 }
 
 template<size_t S, typename T>
@@ -195,7 +237,7 @@ inline uint8_t __TBB_CompareAndSwapGeneric <1,uint8_t> (volatile void *ptr, uint
 #ifdef __TBB_CompareAndSwap1
     return __TBB_CompareAndSwap1(ptr,value,comparand);
 #else
-    return __TBB_MaskedCompareAndSwap<1,uint8_t>((volatile int32_t *)ptr,value,comparand);
+    return __TBB_MaskedCompareAndSwap<1,uint8_t>((volatile uint8_t *)ptr,value,comparand);
 #endif
 }
 
@@ -204,7 +246,7 @@ inline uint16_t __TBB_CompareAndSwapGeneric <2,uint16_t> (volatile void *ptr, ui
 #ifdef __TBB_CompareAndSwap2
     return __TBB_CompareAndSwap2(ptr,value,comparand);
 #else
-    return __TBB_MaskedCompareAndSwap<2,uint16_t>((volatile int32_t *)ptr,value,comparand);
+    return __TBB_MaskedCompareAndSwap<2,uint16_t>((volatile uint16_t *)ptr,value,comparand);
 #endif
 }
 
@@ -220,7 +262,7 @@ inline uint64_t __TBB_CompareAndSwapGeneric <8,uint64_t> (volatile void *ptr, ui
 
 template<size_t S, typename T>
 inline T __TBB_FetchAndAddGeneric (volatile void *ptr, T addend) {
-    AtomicBackoff b;
+    atomic_backoff b;
     T result;
     for(;;) {
         result = *reinterpret_cast<volatile T *>(ptr);
@@ -234,7 +276,7 @@ inline T __TBB_FetchAndAddGeneric (volatile void *ptr, T addend) {
 
 template<size_t S, typename T>
 inline T __TBB_FetchAndStoreGeneric (volatile void *ptr, T value) {
-    AtomicBackoff b;
+    atomic_backoff b;
     T result;
     for(;;) {
         result = *reinterpret_cast<volatile T *>(ptr);
@@ -293,6 +335,33 @@ struct work_around_alignment_bug {
 #endif
 #endif  /* ____TBB_TypeWithAlignmentAtLeastAsStrict */
 
+// Template class here is to avoid instantiation of the static data for modules that don't use it
+template<typename T>
+struct reverse {
+    static const T byte_table[256];
+};
+// An efficient implementation of the reverse function utilizes a 2^8 lookup table holding the bit-reversed
+// values of [0..2^8 - 1]. Those values can also be computed on the fly at a slightly higher cost.
+template<typename T>
+const T reverse<T>::byte_table[256] = {
+    0x00, 0x80, 0x40, 0xC0, 0x20, 0xA0, 0x60, 0xE0, 0x10, 0x90, 0x50, 0xD0, 0x30, 0xB0, 0x70, 0xF0,
+    0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8, 0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78, 0xF8,
+    0x04, 0x84, 0x44, 0xC4, 0x24, 0xA4, 0x64, 0xE4, 0x14, 0x94, 0x54, 0xD4, 0x34, 0xB4, 0x74, 0xF4,
+    0x0C, 0x8C, 0x4C, 0xCC, 0x2C, 0xAC, 0x6C, 0xEC, 0x1C, 0x9C, 0x5C, 0xDC, 0x3C, 0xBC, 0x7C, 0xFC,
+    0x02, 0x82, 0x42, 0xC2, 0x22, 0xA2, 0x62, 0xE2, 0x12, 0x92, 0x52, 0xD2, 0x32, 0xB2, 0x72, 0xF2,
+    0x0A, 0x8A, 0x4A, 0xCA, 0x2A, 0xAA, 0x6A, 0xEA, 0x1A, 0x9A, 0x5A, 0xDA, 0x3A, 0xBA, 0x7A, 0xFA,
+    0x06, 0x86, 0x46, 0xC6, 0x26, 0xA6, 0x66, 0xE6, 0x16, 0x96, 0x56, 0xD6, 0x36, 0xB6, 0x76, 0xF6,
+    0x0E, 0x8E, 0x4E, 0xCE, 0x2E, 0xAE, 0x6E, 0xEE, 0x1E, 0x9E, 0x5E, 0xDE, 0x3E, 0xBE, 0x7E, 0xFE,
+    0x01, 0x81, 0x41, 0xC1, 0x21, 0xA1, 0x61, 0xE1, 0x11, 0x91, 0x51, 0xD1, 0x31, 0xB1, 0x71, 0xF1,
+    0x09, 0x89, 0x49, 0xC9, 0x29, 0xA9, 0x69, 0xE9, 0x19, 0x99, 0x59, 0xD9, 0x39, 0xB9, 0x79, 0xF9,
+    0x05, 0x85, 0x45, 0xC5, 0x25, 0xA5, 0x65, 0xE5, 0x15, 0x95, 0x55, 0xD5, 0x35, 0xB5, 0x75, 0xF5,
+    0x0D, 0x8D, 0x4D, 0xCD, 0x2D, 0xAD, 0x6D, 0xED, 0x1D, 0x9D, 0x5D, 0xDD, 0x3D, 0xBD, 0x7D, 0xFD,
+    0x03, 0x83, 0x43, 0xC3, 0x23, 0xA3, 0x63, 0xE3, 0x13, 0x93, 0x53, 0xD3, 0x33, 0xB3, 0x73, 0xF3,
+    0x0B, 0x8B, 0x4B, 0xCB, 0x2B, 0xAB, 0x6B, 0xEB, 0x1B, 0x9B, 0x5B, 0xDB, 0x3B, 0xBB, 0x7B, 0xFB,
+    0x07, 0x87, 0x47, 0xC7, 0x27, 0xA7, 0x67, 0xE7, 0x17, 0x97, 0x57, 0xD7, 0x37, 0xB7, 0x77, 0xF7,
+    0x0F, 0x8F, 0x4F, 0xCF, 0x2F, 0xAF, 0x6F, 0xEF, 0x1F, 0x9F, 0x5F, 0xDF, 0x3F, 0xBF, 0x7F, 0xFF
+};
+
 } // namespace internal
 } // namespace tbb
 
@@ -489,7 +558,7 @@ struct work_around_alignment_bug {
 // On 32-bit platforms, "atomic.h" requires definition of __TBB_Store8 and __TBB_Load8
 #ifndef __TBB_Store8
 inline void __TBB_Store8 (volatile void *ptr, int64_t value) {
-    tbb::internal::AtomicBackoff b;
+    tbb::internal::atomic_backoff b;
     for(;;) {
         int64_t result = *(int64_t *)ptr;
         if( __TBB_CompareAndSwap8(ptr,value,result)==result ) break;
@@ -509,15 +578,23 @@ inline int64_t __TBB_Load8 (const volatile void *ptr) {
 
 #ifndef __TBB_Log2
 inline intptr_t __TBB_Log2( uintptr_t x ) {
-    long result = -1;
-    for(; x; x>>=1 ) ++result;
-    return result;
+    if( x==0 ) return -1;
+    intptr_t result = 0;
+    uintptr_t tmp;
+#if __TBB_WORDSIZE>=8
+    if( (tmp = x>>32) ) { x=tmp; result += 32; }
+#endif
+    if( (tmp = x>>16) ) { x=tmp; result += 16; }
+    if( (tmp = x>>8) )  { x=tmp; result += 8; }
+    if( (tmp = x>>4) )  { x=tmp; result += 4; }
+    if( (tmp = x>>2) )  { x=tmp; result += 2; }
+    return (x&2)? result+1: result;
 }
 #endif
 
 #ifndef __TBB_AtomicOR
 inline void __TBB_AtomicOR( volatile void *operand, uintptr_t addend ) {
-    tbb::internal::AtomicBackoff b;
+    tbb::internal::atomic_backoff b;
     for(;;) {
         uintptr_t tmp = *(volatile uintptr_t *)operand;
         uintptr_t result = __TBB_CompareAndSwapW(operand, tmp|addend, tmp);
@@ -529,7 +606,7 @@ inline void __TBB_AtomicOR( volatile void *operand, uintptr_t addend ) {
 
 #ifndef __TBB_AtomicAND
 inline void __TBB_AtomicAND( volatile void *operand, uintptr_t addend ) {
-    tbb::internal::AtomicBackoff b;
+    tbb::internal::atomic_backoff b;
     for(;;) {
         uintptr_t tmp = *(volatile uintptr_t *)operand;
         uintptr_t result = __TBB_CompareAndSwapW(operand, tmp&addend, tmp);
@@ -548,7 +625,7 @@ inline bool __TBB_TryLockByte( unsigned char &flag ) {
 #ifndef __TBB_LockByte
 inline uintptr_t __TBB_LockByte( unsigned char& flag ) {
     if ( !__TBB_TryLockByte(flag) ) {
-        tbb::internal::AtomicBackoff b;
+        tbb::internal::atomic_backoff b;
         do {
             b.pause();
         } while ( !__TBB_TryLockByte(flag) );
@@ -557,4 +634,23 @@ inline uintptr_t __TBB_LockByte( unsigned char& flag ) {
 }
 #endif
 
+#ifndef __TBB_ReverseByte
+inline unsigned char __TBB_ReverseByte(unsigned char src) {
+    return tbb::internal::reverse<unsigned char>::byte_table[src];
+}
+#endif
+
+template<typename T>
+T __TBB_ReverseBits(T src)
+{
+    T dst;
+    unsigned char *original = (unsigned char *) &src;
+    unsigned char *reversed = (unsigned char *) &dst;
+
+    for( int i = sizeof(T)-1; i >= 0; i-- )
+        reversed[i] = __TBB_ReverseByte( original[sizeof(T)-i-1] );
+
+    return dst;
+}
+
 #endif /* __TBB_machine_H */
