@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -30,13 +30,23 @@
 #define __TBB_concurrent_vector_H
 
 #include "tbb/tbb_stddef.h"
-#include <iterator>
-#include <new>
 #include "tbb/atomic.h"
 #include "tbb/cache_aligned_allocator.h"
 #include "tbb/blocked_range.h"
-
 #include "tbb/tbb_machine.h"
+#include <new>
+
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+    #pragma warning (push)
+    #pragma warning (disable: 4530)
+#endif
+
+#include <iterator>
+
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    #pragma warning (pop)
+#endif
 
 namespace tbb {
 
@@ -59,8 +69,8 @@ namespace internal {
         static const int min_segment_size = segment_index_t(1)<<lg_min_segment_size;
       
         static segment_index_t segment_index_of( size_t index ) { 
-            uintptr i = index|1<<(lg_min_segment_size-1);
-            uintptr j = __TBB_Log2(i); 
+            uintptr_t i = index|1<<(lg_min_segment_size-1);
+            uintptr_t j = __TBB_Log2(i); 
             return segment_index_t(j-(lg_min_segment_size-1)); 
         }
 
@@ -76,9 +86,9 @@ namespace internal {
 
         typedef size_t size_type;
 
-        void internal_reserve( size_type n, size_type element_size, size_type max_size );
+        void __TBB_EXPORTED_METHOD internal_reserve( size_type n, size_type element_size, size_type max_size );
 
-        size_type internal_capacity() const;
+        size_type __TBB_EXPORTED_METHOD internal_capacity() const;
 
         //! Requested size of vector
         atomic<size_type> my_early_size;
@@ -93,7 +103,7 @@ namespace internal {
             }
 #endif /* TBB_DO_ASSERT */
         };
- 
+
         atomic<segment_t*> my_segment;
 
         segment_t my_storage[2];
@@ -106,18 +116,18 @@ namespace internal {
         }
 
         //! An operation on an n-lement array starting at begin.
-        typedef void(*internal_array_op1)(void* begin, size_type n );
+        typedef void(__TBB_EXPORTED_FUNC *internal_array_op1)(void* begin, size_type n );
 
         //! An operation on n-element destination array and n-element source array.
-        typedef void(*internal_array_op2)(void* dst, const void* src, size_type n );
+        typedef void(__TBB_EXPORTED_FUNC *internal_array_op2)(void* dst, const void* src, size_type n );
 
-        void internal_grow_to_at_least( size_type new_size, size_type element_size, internal_array_op1 init );
+        void __TBB_EXPORTED_METHOD internal_grow_to_at_least( size_type new_size, size_type element_size, internal_array_op1 init );
         void internal_grow( size_type start, size_type finish, size_type element_size, internal_array_op1 init );
-        size_type internal_grow_by( size_type delta, size_type element_size, internal_array_op1 init );
-        void* internal_push_back( size_type element_size, size_type& index ); 
-        void internal_clear( internal_array_op1 destroy, bool reclaim_storage );
-        void internal_copy( const concurrent_vector_base& src, size_type element_size, internal_array_op2 copy );
-        void internal_assign( const concurrent_vector_base& src, size_type element_size, 
+        size_type __TBB_EXPORTED_METHOD internal_grow_by( size_type delta, size_type element_size, internal_array_op1 init );
+        void* __TBB_EXPORTED_METHOD internal_push_back( size_type element_size, size_type& index );
+        void __TBB_EXPORTED_METHOD internal_clear( internal_array_op1 destroy, bool reclaim_storage );
+        void __TBB_EXPORTED_METHOD internal_copy( const concurrent_vector_base& src, size_type element_size, internal_array_op2 copy );
+        void __TBB_EXPORTED_METHOD internal_assign( const concurrent_vector_base& src, size_type element_size,
                               internal_array_op1 destroy, internal_array_op2 assign, internal_array_op2 copy );
 private:
         //! Private functionality that does not cross DLL boundary.
@@ -457,16 +467,16 @@ private:
     T& internal_subscript( size_type index ) const;
 
     //! Construct n instances of T, starting at "begin".
-    static void initialize_array( void* begin, size_type n );
+    static void __TBB_EXPORTED_FUNC initialize_array( void* begin, size_type n );
 
     //! Construct n instances of T, starting at "begin".
-    static void copy_array( void* dst, const void* src, size_type n );
+    static void __TBB_EXPORTED_FUNC copy_array( void* dst, const void* src, size_type n );
 
     //! Assign n instances of T, starting at "begin".
-    static void assign_array( void* dst, const void* src, size_type n );
+    static void __TBB_EXPORTED_FUNC assign_array( void* dst, const void* src, size_type n );
 
     //! Destroy n instances of T, starting at "begin".
-    static void destroy_array( void* begin, size_type n );
+    static void __TBB_EXPORTED_FUNC destroy_array( void* begin, size_type n );
 };
 
 template<typename T>
