@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,19 +26,27 @@
     the GNU General Public License.
 */
 
-#include <cstring>   // for memset()
 #include "tbb/tbb_stddef.h"
 #include "tbb/tbb_machine.h"
+#include "tbb/tbb_exception.h"
 #include "tbb/_concurrent_queue_internal.h"
+#include "concurrent_monitor.h"
 #include "itt_notify.h"
 #include <new>
-#if _WIN32||_WIN64
-#include <windows.h>
-#endif
-using namespace std;
 
-// enable sleep support
-#define __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE 1
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+    #pragma warning (push)
+    #pragma warning (disable: 4530)
+#endif
+
+#include <cstring>   // for memset()
+
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    #pragma warning (pop)
+#endif
+
+using namespace std;
 
 #if defined(_MSC_VER) && defined(_Wp64)
     // Workaround for overzealous compiler warnings in /Wp64 mode
@@ -48,12 +56,6 @@ using namespace std;
 #define RECORD_EVENTS 0
 
 
-#if __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-#if !_WIN32&&!_WIN64
-#include <pthread.h>
-#endif
-#endif
-
 namespace tbb {
 
 namespace internal {
@@ -116,20 +118,17 @@ public:
     }
 };
 
+struct predicate_leq {
+    ticket t;
+    predicate_leq( ticket t_ ) : t(t_) {}
+    bool operator() ( void* p ) const {return (ticket)p<=t;}
+};
+
 //! Internal representation of a ConcurrentQueue.
 /** For efficient, this class has no constructor.  
     The caller is expected to zero-initialize it. */
 class concurrent_queue_rep {
 public:
-#if __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-# if _WIN32||_WIN64
-    typedef HANDLE waitvar_t;
-    typedef CRITICAL_SECTION mutexvar_t;
-# else 
-    typedef pthread_cond_t  waitvar_t;
-    typedef pthread_mutex_t mutexvar_t;
-# endif
-#endif /* __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
 private:
     friend struct micro_queue;
 
@@ -145,38 +144,14 @@ public:
         return k*phi%n_queue;
     }
 
-#if __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
     atomic<ticket> head_counter;
-    waitvar_t  var_wait_for_items;
-    mutexvar_t mtx_items_avail;
+    concurrent_monitor items_avail;
     atomic<size_t> n_invalid_entries;
-    atomic<uint32_t> n_waiting_consumers;
-#if _WIN32||_WIN64
-    uint32_t consumer_wait_generation;
-    uint32_t n_consumers_to_wakeup;
-    char pad1[NFS_MaxLineSize-((sizeof(atomic<ticket>)+sizeof(waitvar_t)+sizeof(mutexvar_t)+sizeof(atomic<size_t>)+sizeof(atomic<uint32_t>)+sizeof(uint32_t)+sizeof(uint32_t))&(NFS_MaxLineSize-1))];
-#else
-    char pad1[NFS_MaxLineSize-((sizeof(atomic<ticket>)+sizeof(waitvar_t)+sizeof(mutexvar_t)+sizeof(atomic<size_t>)+sizeof(atomic<uint32_t>))&(NFS_MaxLineSize-1))];
-#endif
+    char pad1[NFS_MaxLineSize-((sizeof(atomic<ticket>)+sizeof(concurrent_monitor)+sizeof(atomic<size_t>))&(NFS_MaxLineSize-1))];
 
     atomic<ticket> tail_counter;
-    waitvar_t  var_wait_for_slots;
-    mutexvar_t mtx_slots_avail;
-    atomic<uint32_t> n_waiting_producers;
-#if _WIN32||_WIN64
-    uint32_t producer_wait_generation;
-    uint32_t n_producers_to_wakeup;
-    char pad2[NFS_MaxLineSize-((sizeof(atomic<ticket>)+sizeof(waitvar_t)+sizeof(mutexvar_t)+sizeof(atomic<uint32_t>)+sizeof(uint32_t)+sizeof(uint32_t))&(NFS_MaxLineSize-1))];
-#else
-    char pad2[NFS_MaxLineSize-((sizeof(atomic<ticket>)+sizeof(waitvar_t)+sizeof(mutexvar_t)+sizeof(atomic<uint32_t>))&(NFS_MaxLineSize-1))];
-#endif
-#else /* !__TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
-    atomic<ticket> head_counter;
-    atomic<size_t> n_invalid_entries;
-    char pad1[NFS_MaxLineSize-sizeof(atomic<ticket>)-sizeof(atomic<size_t>)];
-    atomic<ticket> tail_counter;
-    char pad2[NFS_MaxLineSize-sizeof(atomic<ticket>)];
-#endif /* __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
+    concurrent_monitor slots_avail;
+    char pad2[NFS_MaxLineSize-((sizeof(atomic<ticket>)+sizeof(concurrent_monitor))&(NFS_MaxLineSize-1))];
     micro_queue array[n_queue];    
 
     micro_queue& choose( ticket k ) {
@@ -204,9 +179,9 @@ void micro_queue::push( const void* item, ticket k, concurrent_queue_base& base
     page* p = NULL;
     size_t index = k/concurrent_queue_rep::n_queue & (base.items_per_page-1);
     if( !index ) {
-        try {
+        __TBB_TRY {
             p = base.allocate_page();
-        } catch (...) {
+        } __TBB_CATCH(...) {
             ++base.my_rep->n_invalid_entries;
             make_invalid( k );
         }
@@ -221,7 +196,7 @@ void micro_queue::push( const void* item, ticket k, concurrent_queue_base& base
             // no memory. throws an exception; assumes concurrent_queue_rep::n_queue>1
             if( tail_counter&0x1 ) {
                 ++base.my_rep->n_invalid_entries;
-                throw bad_last_alloc();
+                throw_exception( eid_bad_last_alloc );
             }
         } while( tail_counter!=k ) ;
     }
@@ -238,16 +213,16 @@ void micro_queue::push( const void* item, ticket k, concurrent_queue_base& base
     }
     ITT_NOTIFY( sync_acquired, p );
 
-    try {
+    __TBB_TRY {
         base.copy_item( *p, index, item );
         ITT_NOTIFY( sync_releasing, p );
         // If no exception was thrown, mark item as present.
-        p->mask |= uintptr(1)<<index;
+        p->mask |= uintptr_t(1)<<index;
         tail_counter += concurrent_queue_rep::n_queue; 
-    } catch (...) {
+    } __TBB_CATCH(...) {
         ++base.my_rep->n_invalid_entries;
         tail_counter += concurrent_queue_rep::n_queue; 
-        throw;
+        __TBB_RETHROW();
     }
 }
 
@@ -261,14 +236,9 @@ bool micro_queue::pop( void* dst, ticket k, concurrent_queue_base& base ) {
     bool success = false; 
     {
         micro_queue_pop_finalizer finalizer( *this, base, k+concurrent_queue_rep::n_queue, index==base.items_per_page-1 ? &p : NULL ); 
-        if( p.mask & uintptr(1)<<index ) {
+        if( p.mask & uintptr_t(1)<<index ) {
             success = true;
-#if DO_ITT_NOTIFY
-            if( ((intptr_t)dst&1) ) {
-                dst = (void*) ((intptr_t)dst&~1);
-                ITT_NOTIFY( sync_acquired, dst );
-            }
-#endif
+            ITT_NOTIFY( sync_acquired, dst );
             ITT_NOTIFY( sync_acquired, head_page );
             base.assign_and_destroy_item( dst, p, index );
             ITT_NOTIFY( sync_releasing, head_page );
@@ -288,7 +258,7 @@ micro_queue& micro_queue::assign( const micro_queue& src, concurrent_queue_base&
     const page* srcp = src.head_page;
     if( srcp ) {
         ticket g_index = head_counter;
-        try {
+        __TBB_TRY {
             size_t n_items  = (tail_counter-head_counter)/concurrent_queue_rep::n_queue;
             size_t index = head_counter/concurrent_queue_rep::n_queue & (base.items_per_page-1);
             size_t end_in_first_page = (index+n_items<base.items_per_page)?(index+n_items):base.items_per_page;
@@ -311,7 +281,7 @@ micro_queue& micro_queue::assign( const micro_queue& src, concurrent_queue_base&
                 cur_page = cur_page->next;
             }
             tail_page = cur_page;
-        } catch (...) {
+        } __TBB_CATCH(...) {
             make_invalid( g_index );
         }
     } else {
@@ -326,7 +296,7 @@ concurrent_queue_base::page* micro_queue::make_copy( concurrent_queue_base& base
     new_page->next = NULL;
     new_page->mask = src_page->mask;
     for( ; begin_in_page!=end_in_page; ++begin_in_page, ++g_index )
-        if( new_page->mask & uintptr(1)<<begin_in_page )
+        if( new_page->mask & uintptr_t(1)<<begin_in_page )
             base.copy_page_item( *new_page, begin_in_page, *src_page, begin_in_page );
     return new_page;
 }
@@ -345,7 +315,7 @@ void micro_queue::make_invalid( ticket k )
             head_page = static_cast<page*>(invalid_page); 
         tail_page = static_cast<page*>(invalid_page);
     }
-    throw;
+    __TBB_RETHROW();
 }
 
 #if _MSC_VER && !defined(__INTEL_COMPILER)
@@ -369,123 +339,24 @@ concurrent_queue_base_v3::concurrent_queue_base_v3( size_t item_size ) {
     __TBB_ASSERT( (size_t)&my_rep->tail_counter % NFS_GetLineSize()==0, "alignment error" );
     __TBB_ASSERT( (size_t)&my_rep->array % NFS_GetLineSize()==0, "alignment error" );
     memset(my_rep,0,sizeof(concurrent_queue_rep));
+    new ( &my_rep->items_avail ) concurrent_monitor();
+    new ( &my_rep->slots_avail ) concurrent_monitor();
     this->item_size = item_size;
-#if __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-#if _WIN32||_WIN64
-    my_rep->var_wait_for_items = CreateEvent( NULL, TRUE/*manual reset*/, FALSE/*not signalled initially*/, NULL);
-    my_rep->var_wait_for_slots = CreateEvent( NULL, TRUE/*manual reset*/, FALSE/*not signalled initially*/, NULL);
-    InitializeCriticalSection( &my_rep->mtx_items_avail );
-    InitializeCriticalSection( &my_rep->mtx_slots_avail );
-#else 
-    // initialize pthread_mutex_t, and pthread_cond_t
-    pthread_mutexattr_t m_attr;
-    pthread_mutexattr_init( &m_attr );
-#if defined(PTHREAD_PRIO_INHERIT) && !__TBB_PRIO_INHERIT_BROKEN
-    pthread_mutexattr_setprotocol( &m_attr, PTHREAD_PRIO_INHERIT );
-#endif
-    pthread_mutex_init( &my_rep->mtx_items_avail, &m_attr );
-    pthread_mutex_init( &my_rep->mtx_slots_avail, &m_attr );
-    pthread_mutexattr_destroy( &m_attr );
-
-    pthread_condattr_t c_attr;
-    pthread_condattr_init( &c_attr );
-    pthread_cond_init( &my_rep->var_wait_for_items, &c_attr );
-    pthread_cond_init( &my_rep->var_wait_for_slots, &c_attr );
-    pthread_condattr_destroy( &c_attr );
-#endif
-#endif /* __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
 }
 
 concurrent_queue_base_v3::~concurrent_queue_base_v3() {
     size_t nq = my_rep->n_queue;
     for( size_t i=0; i<nq; i++ )
         __TBB_ASSERT( my_rep->array[i].tail_page==NULL, "pages were not freed properly" );
-#if __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-# if _WIN32||_WIN64
-    CloseHandle( my_rep->var_wait_for_items );
-    CloseHandle( my_rep->var_wait_for_slots );
-    DeleteCriticalSection( &my_rep->mtx_items_avail );
-    DeleteCriticalSection( &my_rep->mtx_slots_avail );
-# else
-    pthread_mutex_destroy( &my_rep->mtx_items_avail );
-    pthread_mutex_destroy( &my_rep->mtx_slots_avail );
-    pthread_cond_destroy( &my_rep->var_wait_for_items );
-    pthread_cond_destroy( &my_rep->var_wait_for_slots );
-# endif
-#endif /* __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
     cache_aligned_allocator<concurrent_queue_rep>().deallocate(my_rep,1);
 }
 
 void concurrent_queue_base_v3::internal_push( const void* src ) {
     concurrent_queue_rep& r = *my_rep;
-#if !__TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-    ticket k = r.tail_counter++;
-    ptrdiff_t e = my_capacity;
-    if( e<concurrent_queue_rep::infinite_capacity ) {
-        atomic_backoff backoff;
-        for(;;) {
-            if( (ptrdiff_t)(k-r.head_counter)<e ) break;
-            backoff.pause();
-            e = const_cast<volatile ptrdiff_t&>(my_capacity);
-        }
-    } 
-    r.choose(k).push(src,k,*this);
-#elif _WIN32||_WIN64
-    ticket k = r.tail_counter++;
-    ptrdiff_t e = my_capacity;
-    atomic_backoff backoff;
-#if DO_ITT_NOTIFY
-    bool sync_prepare_done = false;
-#endif
-
-    while( (ptrdiff_t)(k-r.head_counter)>=e ) {
-#if DO_ITT_NOTIFY
-        if( !sync_prepare_done ) {
-            ITT_NOTIFY( sync_prepare, &sync_prepare_done );
-            sync_prepare_done = true;
-        }
-#endif
-        if( !backoff.bounded_pause() ) {
-            EnterCriticalSection( &r.mtx_slots_avail );
-            r.n_waiting_producers++;
-            while( (ptrdiff_t)(k-r.head_counter)>=const_cast<volatile ptrdiff_t&>(my_capacity) ) {
-                uint32_t my_generation = r.producer_wait_generation;
-                for( ;; ) {
-                    LeaveCriticalSection( &r.mtx_slots_avail );
-                    WaitForSingleObject( r.var_wait_for_slots, INFINITE );
-                    EnterCriticalSection( &r.mtx_slots_avail );
-                    if( r.n_producers_to_wakeup > 0 && r.producer_wait_generation != my_generation )
-                        break;
-                }
-                if( --r.n_producers_to_wakeup == 0 )
-                    ResetEvent( r.var_wait_for_slots );
-            }
-            --r.n_waiting_producers;
-            LeaveCriticalSection( &r.mtx_slots_avail );
-            break;
-        }
-        e = const_cast<volatile ptrdiff_t&>(my_capacity);
-    }
-#if DO_ITT_NOTIFY
-    if( sync_prepare_done )
-        ITT_NOTIFY( sync_acquired, &sync_prepare_done );
-#endif
-
-    r.choose( k ).push( src, k, *this );
-
-    if( r.n_waiting_consumers>0 ) {
-        EnterCriticalSection( &r.mtx_items_avail );
-        if( r.n_waiting_consumers>0 ) {
-            r.consumer_wait_generation++;
-            r.n_consumers_to_wakeup = r.n_waiting_consumers;
-            SetEvent( r.var_wait_for_items );
-        }
-        LeaveCriticalSection( &r.mtx_items_avail );
-    }
-#else 
     ticket k = r.tail_counter++;
     ptrdiff_t e = my_capacity;
     atomic_backoff backoff;
+    concurrent_monitor::thread_context thr_ctx;
 #if DO_ITT_NOTIFY
     bool sync_prepare_done = false;
 #endif
@@ -497,44 +368,29 @@ void concurrent_queue_base_v3::internal_push( const void* src ) {
         }
 #endif
         if( !backoff.bounded_pause() ) {
-            // queue is full.  go to sleep. let them go to sleep in order.
-            pthread_mutex_lock( &r.mtx_slots_avail );
-            r.n_waiting_producers++;
-            while( (ptrdiff_t)(k-r.head_counter)>=const_cast<volatile ptrdiff_t&>(my_capacity) ) {
-                pthread_cond_wait( &r.var_wait_for_slots, &r.mtx_slots_avail );
+            bool slept = false;
+            r.slots_avail.prepare_wait( thr_ctx, (void*) ((ptrdiff_t)(k-e)) );
+            while( (ptrdiff_t)(k-r.head_counter)>=const_cast<volatile ptrdiff_t&>(e = my_capacity) ) {
+                if( (slept = r.slots_avail.commit_wait( thr_ctx ) )==true )
+                    break;
+                r.slots_avail.prepare_wait( thr_ctx, (void*) ((ptrdiff_t)(k-e)) );
             }
-            --r.n_waiting_producers;
-            pthread_mutex_unlock( &r.mtx_slots_avail );
+            if( !slept )
+                r.slots_avail.cancel_wait( thr_ctx );
             break;
         }
         e = const_cast<volatile ptrdiff_t&>(my_capacity);
     }
-#if DO_ITT_NOTIFY
-    if( sync_prepare_done )
-        ITT_NOTIFY( sync_acquired, &sync_prepare_done );
-#endif
+    ITT_NOTIFY( sync_acquired, &sync_prepare_done );
     r.choose( k ).push( src, k, *this );
-
-    if( r.n_waiting_consumers>0 ) {
-        pthread_mutex_lock( &r.mtx_items_avail );
-        // pthread_cond_broadcast() wakes up all consumers. 
-        if( r.n_waiting_consumers>0 )
-            pthread_cond_broadcast( &r.var_wait_for_items );
-        pthread_mutex_unlock( &r.mtx_items_avail );
-    }
-#endif /* !__TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
+    r.items_avail.notify( predicate_leq(k) );
 }
 
 void concurrent_queue_base_v3::internal_pop( void* dst ) {
     concurrent_queue_rep& r = *my_rep;
-#if !__TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-    ticket k;
-    do {
-        k = r.head_counter++;
-    } while( !r.choose(k).pop(dst,k,*this) );
-#elif _WIN32||_WIN64
     ticket k;
     atomic_backoff backoff;
+    concurrent_monitor::thread_context thr_ctx;
 #if DO_ITT_NOTIFY
     bool sync_prepare_done = false;
 #endif
@@ -544,83 +400,27 @@ void concurrent_queue_base_v3::internal_pop( void* dst ) {
 #if DO_ITT_NOTIFY
             if( !sync_prepare_done ) {
                 ITT_NOTIFY( sync_prepare, dst );
-                dst = (void*) ((intptr_t)dst | 1);
                 sync_prepare_done = true;
             }
 #endif
             // Queue is empty; pause and re-try a few times
             if( !backoff.bounded_pause() ) {
-                // it is really empty.. go to sleep
-                EnterCriticalSection( &r.mtx_items_avail );
-                r.n_waiting_consumers++;
+                bool slept = false;
+                r.items_avail.prepare_wait( thr_ctx, (void*)k );
                 while( r.tail_counter<=k ) {
-                    uint32_t my_generation = r.consumer_wait_generation;
-                    for( ;; ) {
-                        LeaveCriticalSection( &r.mtx_items_avail );
-                        WaitForSingleObject( r.var_wait_for_items, INFINITE );
-                        EnterCriticalSection( &r.mtx_items_avail );
-                        if( r.n_consumers_to_wakeup > 0 && r.consumer_wait_generation != my_generation )
-                            break;
-                    }
-                    if( --r.n_consumers_to_wakeup == 0 )
-                        ResetEvent( r.var_wait_for_items );
+                    if( (slept = r.items_avail.commit_wait( thr_ctx ) )==true )
+                        break;
+                    r.items_avail.prepare_wait( thr_ctx, (void*)k );
                 }
-                --r.n_waiting_consumers;
-                LeaveCriticalSection( &r.mtx_items_avail );
-                backoff.reset();
+                if( !slept )
+                    r.items_avail.cancel_wait( thr_ctx );
                 break; // break from inner while
             }
         } // break to here
     } while( !r.choose(k).pop(dst,k,*this) );
 
     // wake up a producer..
-    if( r.n_waiting_producers>0 ) {
-        EnterCriticalSection( &r.mtx_slots_avail );
-        if( r.n_waiting_producers>0 ) {
-            r.producer_wait_generation++;
-            r.n_producers_to_wakeup = r.n_waiting_producers;
-            SetEvent( r.var_wait_for_slots );
-        }
-        LeaveCriticalSection( &r.mtx_slots_avail );
-    }
-#else 
-    ticket k;
-    atomic_backoff backoff;
-#if DO_ITT_NOTIFY
-    bool sync_prepare_done = false;
-#endif
-    do {
-        k = r.head_counter++;
-        while( r.tail_counter<=k ) {
-#if DO_ITT_NOTIFY
-            if( !sync_prepare_done ) {
-                ITT_NOTIFY( sync_prepare, dst );
-                dst = (void*) ((intptr_t)dst | 1);
-                sync_prepare_done = true;
-            }
-#endif
-            // Queue is empty; pause and re-try a few times
-            if( !backoff.bounded_pause() ) {
-                // it is really empty.. go to sleep
-                pthread_mutex_lock( &r.mtx_items_avail );
-                r.n_waiting_consumers++;
-                while( r.tail_counter<=k )
-                    pthread_cond_wait( &r.var_wait_for_items, &r.mtx_items_avail );
-                --r.n_waiting_consumers;
-                pthread_mutex_unlock( &r.mtx_items_avail );
-                backoff.reset();
-                break;
-            }
-        }
-    } while( !r.choose(k).pop(dst,k,*this) );
-
-    if( r.n_waiting_producers>0 ) {
-        pthread_mutex_lock( &r.mtx_slots_avail );
-        if( r.n_waiting_producers>0 )
-            pthread_cond_broadcast( &r.var_wait_for_slots );
-        pthread_mutex_unlock( &r.mtx_slots_avail );
-    }
-#endif /* !__TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
+    r.slots_avail.notify( predicate_leq(k) );
 }
 
 bool concurrent_queue_base_v3::internal_pop_if_present( void* dst ) {
@@ -642,27 +442,7 @@ bool concurrent_queue_base_v3::internal_pop_if_present( void* dst ) {
         }
     } while( !r.choose( k ).pop( dst, k, *this ) );
 
-#if __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-#if _WIN32||_WIN64
-    // wake up a producer..
-    if( r.n_waiting_producers>0 ) {
-        EnterCriticalSection( &r.mtx_slots_avail );
-        if( r.n_waiting_producers>0 ) {
-            r.producer_wait_generation++;
-            r.n_producers_to_wakeup = r.n_waiting_producers;
-            SetEvent( r.var_wait_for_slots );
-        }
-        LeaveCriticalSection( &r.mtx_slots_avail );
-    }
-#else /* including MacOS */
-    if( r.n_waiting_producers>0 ) {
-        pthread_mutex_lock( &r.mtx_slots_avail );
-        if( r.n_waiting_producers>0 )
-            pthread_cond_broadcast( &r.var_wait_for_slots );
-        pthread_mutex_unlock( &r.mtx_slots_avail );
-    }
-#endif
-#endif /* __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
+    r.slots_avail.notify( predicate_leq(k) );
 
     return true;
 }
@@ -684,26 +464,7 @@ bool concurrent_queue_base_v3::internal_push_if_not_full( const void* src ) {
     }
     r.choose(k).push(src,k,*this);
 
-#if __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE
-#if _WIN32||_WIN64
-    if( r.n_waiting_consumers>0 ) {
-        EnterCriticalSection( &r.mtx_items_avail );
-        if( r.n_waiting_consumers>0 ) {
-            r.consumer_wait_generation++;
-            r.n_consumers_to_wakeup = r.n_waiting_consumers;
-            SetEvent( r.var_wait_for_items );
-        }
-        LeaveCriticalSection( &r.mtx_items_avail );
-    }
-#else /* including MacOS */
-    if( r.n_waiting_consumers>0 ) {
-        pthread_mutex_lock( &r.mtx_items_avail );
-        if( r.n_waiting_consumers>0 )
-            pthread_cond_broadcast( &r.var_wait_for_items );
-        pthread_mutex_unlock( &r.mtx_items_avail );
-    }
-#endif
-#endif /* __TBB_NO_BUSY_WAIT_IN_CONCURRENT_QUEUE */
+    r.items_avail.notify( predicate_leq(k) );
     return true;
 }
 
@@ -736,7 +497,7 @@ void concurrent_queue_base_v3::internal_finish_clear() {
 }
 
 void concurrent_queue_base_v3::internal_throw_exception() const {
-    throw bad_alloc();
+    throw_exception( eid_bad_alloc );
 }
 
 void concurrent_queue_base_v3::assign( const concurrent_queue_base& src ) {
@@ -763,10 +524,12 @@ class concurrent_queue_iterator_rep: no_assign {
 public:
     ticket head_counter;   
     const concurrent_queue_base& my_queue;
+    const size_t offset_of_last;
     concurrent_queue_base::page* array[concurrent_queue_rep::n_queue];
-    concurrent_queue_iterator_rep( const concurrent_queue_base& queue ) : 
+    concurrent_queue_iterator_rep( const concurrent_queue_base& queue, size_t offset_of_last_ ) : 
         head_counter(queue.my_rep->head_counter),
-        my_queue(queue)
+        my_queue(queue),
+        offset_of_last(offset_of_last_)
     {
         const concurrent_queue_rep& rep = *queue.my_rep;
         for( size_t k=0; k<concurrent_queue_rep::n_queue; ++k )
@@ -781,8 +544,8 @@ public:
             concurrent_queue_base::page* p = array[concurrent_queue_rep::index(k)];
             __TBB_ASSERT(p,NULL);
             size_t i = k/concurrent_queue_rep::n_queue & (my_queue.items_per_page-1);
-            item = static_cast<unsigned char*>(static_cast<void*>(p+1)) + my_queue.item_size*i;
-            return (p->mask & uintptr(1)<<i)!=0;
+            item = static_cast<unsigned char*>(static_cast<void*>(p)) + offset_of_last + my_queue.item_size*i;
+            return (p->mask & uintptr_t(1)<<i)!=0;
         }
     }
 };
@@ -790,13 +553,22 @@ public:
 //------------------------------------------------------------------------
 // concurrent_queue_iterator_base
 //------------------------------------------------------------------------
-concurrent_queue_iterator_base_v3::concurrent_queue_iterator_base_v3( const concurrent_queue_base& queue ) {
+
+void concurrent_queue_iterator_base_v3::initialize( const concurrent_queue_base& queue, size_t offset_of_last ) {
     my_rep = cache_aligned_allocator<concurrent_queue_iterator_rep>().allocate(1);
-    new( my_rep ) concurrent_queue_iterator_rep(queue);
+    new( my_rep ) concurrent_queue_iterator_rep(queue,offset_of_last);
     size_t k = my_rep->head_counter;
     if( !my_rep->get_item(my_item, k) ) advance();
 }
 
+concurrent_queue_iterator_base_v3::concurrent_queue_iterator_base_v3( const concurrent_queue_base& queue ) {
+    initialize(queue,0);
+}
+
+concurrent_queue_iterator_base_v3::concurrent_queue_iterator_base_v3( const concurrent_queue_base& queue, size_t offset_of_last ) {
+    initialize(queue,offset_of_last);
+}
+
 void concurrent_queue_iterator_base_v3::assign( const concurrent_queue_iterator_base& other ) {
     if( my_rep!=other.my_rep ) {
         if( my_rep ) {
