@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -30,6 +30,8 @@
 #include "tbb/spin_mutex.h"
 #include "tbb/cache_aligned_allocator.h"
 #include "itt_notify.h"
+#include "semaphore.h"
+#include "tls.h"  // for parallel filters that do not use NULL as end_of_input
 
 
 namespace tbb {
@@ -57,13 +59,19 @@ struct task_info {
 /** Each item is a task_info, inserted into a position in the buffer corresponding to a Token. */
 class input_buffer {
     friend class tbb::internal::pipeline_root_task;
+    friend class tbb::filter;
     friend class tbb::thread_bound_filter;
+    friend class tbb::internal::stage_task;
+    friend class tbb::pipeline;
 
     typedef  Token  size_type;
 
     //! Array of deferred tasks that cannot yet start executing. 
     task_info* array;
 
+    //! for thread-bound filter, semaphore for waiting, NULL otherwise.
+    semaphore* my_sem;
+
     //! Size of array
     /** Always 0 or a power of 2 */
     size_type array_size;
@@ -83,7 +91,7 @@ class input_buffer {
     /** Must be a power of 2 */
     static const size_type initial_buffer_size = 4;
 
-    //! Used only for out of order buffer.
+    //! Used for out of order buffer, and for assigning my_token if is_ordered and my_token not already assigned
     Token high_token;
 
     //! True for ordered filter, false otherwise. 
@@ -91,14 +99,27 @@ class input_buffer {
 
     //! True for thread-bound filter, false otherwise. 
     bool is_bound;
+
+    //! for parallel filters that accepts NULLs, thread-local flag for reaching end_of_input
+    typedef basic_tls<intptr_t> end_of_input_tls_t;
+    end_of_input_tls_t end_of_input_tls;
+    bool end_of_input_tls_allocated; // no way to test pthread creation of TLS
+
+    void create_sema(size_t initial_tokens) { __TBB_ASSERT(!my_sem,NULL); my_sem = new internal::semaphore(initial_tokens); }
+    void free_sema() { __TBB_ASSERT(my_sem,NULL); delete my_sem; }
+    void sema_P() { __TBB_ASSERT(my_sem,NULL); my_sem->P(); }
+    void sema_V() { __TBB_ASSERT(my_sem,NULL); my_sem->V(); }
+
 public:
     //! Construct empty buffer.
     input_buffer( bool is_ordered_, bool is_bound_ ) : 
-            array(NULL), array_size(0),
+            array(NULL), my_sem(NULL), array_size(0),
             low_token(0), high_token(0), 
-            is_ordered(is_ordered_), is_bound(is_bound_) {
+            is_ordered(is_ordered_), is_bound(is_bound_),
+            end_of_input_tls_allocated(false) {
         grow(initial_buffer_size);
         __TBB_ASSERT( array, NULL );
+        if(is_bound) create_sema(0);
     }
 
     //! Destroy the buffer.
@@ -106,34 +127,48 @@ public:
         __TBB_ASSERT( array, NULL );
         cache_aligned_allocator<task_info>().deallocate(array,array_size);
         poison_pointer( array );
+        if(my_sem) {
+            free_sema();
+        }
+        if(end_of_input_tls_allocated) {
+            destroy_my_tls();
+        }
     }
 
     //! Put a token into the buffer.
     /** If task information was placed into buffer, returns true;
         otherwise returns false, informing the caller to create and spawn a task.
+        If input buffer owned by thread-bound filter and the item at
+        low_token was not valid, issue a V()
+        If the input_buffer is owned by a successor to a thread-bound filter,
+        the force_put parameter should be true to ensure the token is inserted
+        in the buffer.
     */
-    // Using template to avoid explicit dependency on stage_task
-    template<typename StageTask>
-    bool put_token( StageTask& putter ) {
+    bool put_token( task_info& info_, bool force_put = false ) {
         {
+            info_.is_valid = true;
             spin_mutex::scoped_lock lock( array_mutex );
             Token token;
+            bool was_empty = !array[low_token&(array_size-1)].is_valid;
             if( is_ordered ) {
-                if( !putter.my_token_ready ) {
-                    putter.my_token = high_token++;
-                    putter.my_token_ready = true;
+                if( !info_.my_token_ready ) {
+                    info_.my_token = high_token++;
+                    info_.my_token_ready = true;
                 }
-                token = putter.my_token;
+                token = info_.my_token;
             } else
                 token = high_token++;
             __TBB_ASSERT( (tokendiff_t)(token-low_token)>=0, NULL );
-            if( token!=low_token || is_bound ) {
+            if( token!=low_token || is_bound || force_put ) {
                 // Trying to put token that is beyond low_token.
                 // Need to wait until low_token catches up before dispatching.
                 if( token-low_token>=array_size ) 
                     grow( token-low_token+1 );
                 ITT_NOTIFY( sync_releasing, this );
-                putter.put_task_info(array[token&array_size-1]);
+                array[token&(array_size-1)] = info_;
+                if(was_empty && is_bound) {
+                    sema_V();
+                }
                 return true;
             }
         }
@@ -143,6 +178,10 @@ public:
     //! Note that processing of a token is finished.
     /** Fires up processing of the next token, if processing was deferred. */
     // Using template to avoid explicit dependency on stage_task
+    // this is only called for serial filters, and is the reason for the
+    // advance parameter in return_item (we're incrementing low_token here.)
+    // Non-TBF serial stages don't advance the token at the start because the presence
+    // of the current token in the buffer keeps another stage from being spawned.
     template<typename StageTask>
     void note_done( Token token, StageTask& spawner ) {
         task_info wakee;
@@ -151,7 +190,7 @@ public:
             spin_mutex::scoped_lock lock( array_mutex );
             if( !is_ordered || token==low_token ) {
                 // Wake the next task
-                task_info& item = array[++low_token & array_size-1];
+                task_info& item = array[++low_token & (array_size-1)];
                 ITT_NOTIFY( sync_acquired, this );
                 wakee = item;
                 item.is_valid = false;
@@ -161,12 +200,12 @@ public:
             spawner.spawn_stage_task(wakee);
     }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     //! The method destroys all data in filters to prevent memory leaks
     void clear( filter* my_filter ) {
         long t=low_token;
         for( size_type i=0; i<array_size; ++i, ++t ){
-            task_info& temp = array[t&array_size-1];
+            task_info& temp = array[t&(array_size-1)];
             if (temp.is_valid ) {
                 my_filter->finalize(temp.my_object);
                 temp.is_valid = false;
@@ -175,9 +214,12 @@ public:
     }
 #endif
 
+    //! return an item, invalidate the queued item, but only advance if advance
+    //  advance == true for parallel filters.  If the filter is serial, leave the
+    // item in the buffer to keep another stage from being spawned.
     bool return_item(task_info& info, bool advance) {
         spin_mutex::scoped_lock lock( array_mutex );
-        task_info& item = array[low_token&array_size-1];
+        task_info& item = array[low_token&(array_size-1)];
         ITT_NOTIFY( sync_acquired, this );
         if( item.is_valid ) {
             info = item;
@@ -188,24 +230,14 @@ public:
         return false;
     }
 
-    void put_item( task_info& info ) {
-        info.is_valid = true;
-        spin_mutex::scoped_lock lock( array_mutex );
-        Token token;
-        if( is_ordered ) {
-            if( !info.my_token_ready ) {
-                info.my_token = high_token++;
-                info.my_token_ready = true;
-            }
-            token = info.my_token;
-        } else
-            token = high_token++;
-        __TBB_ASSERT( (tokendiff_t)(token-low_token)>=0, NULL );
-        if( token-low_token>=array_size ) 
-            grow( token-low_token+1 );
-        ITT_NOTIFY( sync_releasing, this );
-        array[token&array_size-1] = info;
-    }
+    //! true if the current low_token is valid.
+    bool has_item() { spin_mutex::scoped_lock lock(array_mutex); return array[low_token&(array_size -1)].is_valid; }
+
+    // end_of_input signal for parallel_pipeline, parallel input filters with 0 tokens allowed.
+    void create_my_tls() { int status = end_of_input_tls.create(); if(status) handle_perror(status, "TLS not allocated for filter"); end_of_input_tls_allocated = true; }
+    void destroy_my_tls() { int status = end_of_input_tls.destroy(); if(status) handle_perror(status, "Failed to destroy filter TLS"); }
+    bool my_tls_end_of_input() { return end_of_input_tls.get() != 0; }
+    void set_my_tls_end_of_input() { end_of_input_tls.set(1); }
 };
 
 void input_buffer::grow( size_type minimum_size ) {
@@ -219,7 +251,7 @@ void input_buffer::grow( size_type minimum_size ) {
         new_array[i].is_valid = false;
     long t=low_token;
     for( size_type i=0; i<old_size; ++i, ++t )
-        new_array[t&new_size-1] = old_array[t&old_size-1];
+        new_array[t&(new_size-1)] = old_array[t&(old_size-1)];
     array = new_array;
     array_size = new_size;
     if( old_array )
@@ -233,6 +265,7 @@ private:
     filter* my_filter;  
     //! True if this task has not yet read the input.
     bool my_at_start;
+
 public:
     //! Construct stage_task for first stage in a pipeline.
     /** Such a stage has not read any input yet. */
@@ -258,7 +291,7 @@ public:
     }
     //! The virtual task execution method
     /*override*/ task* execute();
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     ~stage_task()    
     {
         if (my_filter && my_object && (my_filter->my_filter_mode & filter::version_mask) >= __TBB_PIPELINE_VERSION(4)) {
@@ -267,7 +300,7 @@ public:
             my_object = NULL;
         }
     }
-#endif // __TBB_EXCEPTIONS
+#endif // __TBB_TASK_GROUP_CONTEXT
     //! Creates and spawns stage_task from task_info
     void spawn_stage_task(const task_info& info)
     {
@@ -275,13 +308,6 @@ public:
                                 stage_task( my_pipeline, my_filter, info );
         spawn(*clone);
     }
-    //! Puts current task information
-    void put_task_info(task_info &where_to_put ) {
-        where_to_put.my_object = my_object;
-        where_to_put.my_token = my_token;
-        where_to_put.my_token_ready = my_token_ready;
-        where_to_put.is_valid = true;
-    }
 };
 
 task* stage_task::execute() {
@@ -290,7 +316,8 @@ task* stage_task::execute() {
     if( my_at_start ) {
         if( my_filter->is_serial() ) {
             my_object = (*my_filter)(my_object);
-            if( my_object ) {
+            if( my_object || ( my_filter->object_may_be_null() && !my_pipeline.end_of_input) ) 
+            {
                 if( my_filter->is_ordered() ) {
                     my_token = my_pipeline.token_counter++; // ideally, with relaxed semantics
                     my_token_ready = true;
@@ -298,7 +325,7 @@ task* stage_task::execute() {
                     if( my_pipeline.has_thread_bound_filters )
                         my_pipeline.token_counter++; // ideally, with relaxed semantics
                 }
-                if( !my_filter->next_filter_in_pipeline ) {
+                if( !my_filter->next_filter_in_pipeline ) { // we're only filter in pipeline
                     reset();
                     goto process_another_stage;
                 } else {
@@ -321,11 +348,12 @@ task* stage_task::execute() {
             if( --my_pipeline.input_tokens>0 )
                 spawn( *new( allocate_additional_child_of(*parent()) ) stage_task( my_pipeline ) );
             my_object = (*my_filter)(my_object);
-            if( !my_object ) {
+            if( !my_object && (!my_filter->object_may_be_null() || my_filter->my_input_buffer->my_tls_end_of_input()) )
+            { 
                 my_pipeline.end_of_input = true; 
                 if( (my_filter->my_filter_mode & my_filter->version_mask) >= __TBB_PIPELINE_VERSION(5) ) {
                     if( my_pipeline.has_thread_bound_filters )
-                        my_pipeline.token_counter--;
+                        my_pipeline.token_counter--;  // fix token_counter
                 }
                 return NULL;
             }
@@ -351,7 +379,7 @@ task* stage_task::execute() {
                         my_filter = my_filter->next_filter_in_pipeline;
                     } while( my_filter && my_filter->is_bound() );
                     // Check if there is an item ready to process
-                    if( my_filter && my_filter->my_input_buffer->return_item(*this, !my_filter->is_serial()) ) 
+                    if( my_filter && my_filter->my_input_buffer->return_item(*this, !my_filter->is_serial()))
                         goto process_another_stage;
                 } 
                 my_filter = NULL; // To prevent deleting my_object twice if exception occurs
@@ -360,8 +388,17 @@ task* stage_task::execute() {
         }
     } else {
         // Reached end of the pipe.
-        if( ++my_pipeline.input_tokens>1 || my_pipeline.end_of_input || my_pipeline.filter_list->is_bound() )
+        size_t ntokens_avail = ++my_pipeline.input_tokens;
+        if(my_pipeline.filter_list->is_bound() ) {
+            if(ntokens_avail == 1) {
+                my_pipeline.filter_list->my_input_buffer->sema_V();
+            }
+            return NULL;
+        }
+        if( ntokens_avail>1  // Only recycle if there is one available token
+                || my_pipeline.end_of_input ) {
             return NULL; // No need to recycle for new input
+        }
         ITT_NOTIFY( sync_acquired, &my_pipeline.input_tokens );
         // Recycle as an input stage task.
         reset();
@@ -394,8 +431,7 @@ class pipeline_root_task: public task {
             while( current_filter ) {
                 __TBB_ASSERT( !current_filter->is_bound(), "filter is thread-bound?" );
                 __TBB_ASSERT( current_filter->prev_filter_in_pipeline->is_bound(), "previous filter is not thread-bound?" );
-                if( !my_pipeline.end_of_input
-                    || (tokendiff_t)(my_pipeline.token_counter - current_filter->my_input_buffer->low_token) > 0 )
+                if( !my_pipeline.end_of_input || current_filter->has_more_work())
                 {
                     task_info info;
                     info.reset();
@@ -419,7 +455,7 @@ class pipeline_root_task: public task {
                     first_suitable_filter = first_suitable_filter->next_segment;
                     current_filter = first_suitable_filter; 
                 }
-            } /* end of while */
+            } /* while( current_filter ) */
             return NULL;
         } else { 
             if( !my_pipeline.end_of_input ) {
@@ -465,7 +501,7 @@ public:
         my_pipeline(_pipeline)
     {}
     ~pipeline_cleaner(){
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
         if (my_pipeline.end_counter->is_cancelled()) // Pipeline was cancelled
             my_pipeline.clear_filters(); 
 #endif
@@ -479,7 +515,7 @@ void pipeline::inject_token( task& ) {
     __TBB_ASSERT(0,"illegal call to inject_token");
 }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
 void pipeline::clear_filters() {
     for( filter* f = filter_list; f; f = f->next_filter_in_pipeline ) {
         if ((f->my_filter_mode & filter::version_mask) >= __TBB_PIPELINE_VERSION(4))
@@ -555,9 +591,20 @@ void pipeline::add_filter( filter& filter_ ) {
                 has_thread_bound_filters = true;
             filter_.my_input_buffer = new internal::input_buffer( filter_.is_ordered(), filter_.is_bound() );
         }
-        else {
-            if( filter_.prev_filter_in_pipeline && filter_.prev_filter_in_pipeline->is_bound() )
-                filter_.my_input_buffer = new internal::input_buffer( false, false );
+        else { 
+            if(filter_.prev_filter_in_pipeline) {
+                if(filter_.prev_filter_in_pipeline->is_bound()) {
+                    // successors to bound filters must have an input_buffer
+                    filter_.my_input_buffer = new internal::input_buffer( /*is_ordered*/false, false );
+                }
+            }
+            else {  // input filter
+                if(filter_.object_may_be_null() ) {
+                    //TODO: buffer only needed to hold TLS; could improve
+                    filter_.my_input_buffer = new internal::input_buffer( /*is_ordered*/false, false );
+                    filter_.my_input_buffer->create_my_tls();
+                }
+            }
         }
     } else {
         if( filter_.is_serial() ) {
@@ -568,6 +615,9 @@ void pipeline::add_filter( filter& filter_ ) {
 }
 
 void pipeline::remove_filter( filter& filter_ ) {
+    __TBB_ASSERT( filter_.prev_filter_in_pipeline!=filter::not_in_pipeline(), "filter not part of pipeline" );
+    __TBB_ASSERT( filter_.next_filter_in_pipeline!=filter::not_in_pipeline(), "filter not part of pipeline" );
+    __TBB_ASSERT( !end_counter, "invocation of remove_filter on running pipeline" );
     if (&filter_ == filter_list) 
         filter_list = filter_.next_filter_in_pipeline;
     else {
@@ -591,7 +641,7 @@ void pipeline::remove_filter( filter& filter_ ) {
 }
 
 void pipeline::run( size_t max_number_of_live_tokens
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     , tbb::task_group_context& context
 #endif
     ) {
@@ -600,36 +650,75 @@ void pipeline::run( size_t max_number_of_live_tokens
     if( filter_list ) {
         internal::pipeline_cleaner my_pipeline_cleaner(*this);
         end_of_input = false;
-#if __TBB_EXCEPTIONS            
+        input_tokens = internal::Token(max_number_of_live_tokens);
+        if(has_thread_bound_filters) {
+            // release input filter if thread-bound
+            if(filter_list->is_bound()) {
+                filter_list->my_input_buffer->sema_V();
+            }
+        }
+#if __TBB_TASK_GROUP_CONTEXT            
         end_counter = new( task::allocate_root(context) ) internal::pipeline_root_task( *this );
 #else
         end_counter = new( task::allocate_root() ) internal::pipeline_root_task( *this );
 #endif
-        input_tokens = internal::Token(max_number_of_live_tokens);
         // Start execution of tasks
         task::spawn_root_and_wait( *end_counter );
-    } 
+
+        if(has_thread_bound_filters) {
+            for(filter* f = filter_list->next_filter_in_pipeline; f; f=f->next_filter_in_pipeline) {
+                if(f->is_bound()) {
+                    f->my_input_buffer->sema_V(); // wake to end
+                }
+            }
+        }
+    }
 }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
 void pipeline::run( size_t max_number_of_live_tokens ) {
-    tbb::task_group_context context;
-    run(max_number_of_live_tokens, context);
+    if( filter_list ) {
+        // Construct task group context with the exception propagation mode expected 
+        // by the pipeline caller.
+        uintptr_t ctx_traits = filter_list->my_filter_mode & filter::exact_exception_propagation ? 
+                task_group_context::default_traits :
+                task_group_context::default_traits & ~task_group_context::exact_exception;
+        task_group_context context(task_group_context::bound, ctx_traits);
+        run(max_number_of_live_tokens, context);
+    }
+}
+#endif // __TBB_TASK_GROUP_CONTEXT
+
+bool filter::has_more_work() {
+    __TBB_ASSERT(my_pipeline, NULL);
+    __TBB_ASSERT(my_input_buffer, "has_more_work() called for filter with no input buffer");
+    return (internal::tokendiff_t)(my_pipeline->token_counter - my_input_buffer->low_token) != 0;
 }
-#endif // __TBB_EXCEPTIONS
 
 filter::~filter() {
     if ( (my_filter_mode & version_mask) >= __TBB_PIPELINE_VERSION(3) ) {
-        if ( next_filter_in_pipeline != filter::not_in_pipeline() ) { 
-            __TBB_ASSERT( prev_filter_in_pipeline != filter::not_in_pipeline(), "probably filter list is broken" );
+        if ( next_filter_in_pipeline != filter::not_in_pipeline() )
             my_pipeline->remove_filter(*this);
-        } else 
+        else 
             __TBB_ASSERT( prev_filter_in_pipeline == filter::not_in_pipeline(), "probably filter list is broken" );
     } else {
         __TBB_ASSERT( next_filter_in_pipeline==filter::not_in_pipeline(), "cannot destroy filter that is part of pipeline" );
     }
 }
 
+void
+filter::set_end_of_input() {
+    __TBB_ASSERT(my_input_buffer, NULL);
+    __TBB_ASSERT(object_may_be_null(), NULL);
+    if(is_serial()) {
+        my_pipeline->end_of_input = true;
+    }
+    else {
+        __TBB_ASSERT(my_input_buffer->end_of_input_tls_allocated, NULL);
+        my_input_buffer->set_my_tls_end_of_input();
+    }
+}
+
 thread_bound_filter::result_type thread_bound_filter::process_item() {
     return internal_process_item(true);
 }
@@ -641,18 +730,21 @@ thread_bound_filter::result_type thread_bound_filter::try_process_item() {
 thread_bound_filter::result_type thread_bound_filter::internal_process_item(bool is_blocking) {
     internal::task_info info;
     info.reset();
-    
+
+    if(my_pipeline && my_pipeline->end_of_input && !has_more_work())
+        return end_of_stream;
+
     if( !prev_filter_in_pipeline ) {
         if( my_pipeline->end_of_input )
             return end_of_stream;
-        while( my_pipeline->input_tokens == 0 ) {
-            if( is_blocking )
-                __TBB_Yield();
-            else
+        while(my_pipeline->input_tokens == 0) {
+            if( !is_blocking ) 
                 return item_not_available;
+            my_input_buffer->sema_P();
         }
         info.my_object = (*this)(info.my_object);
         if( info.my_object ) {
+            __TBB_ASSERT(my_pipeline->input_tokens > 0, "Token failed in thread-bound filter");
             my_pipeline->input_tokens--;
             if( is_ordered() ) {
                 info.my_token = my_pipeline->token_counter;
@@ -664,20 +756,31 @@ thread_bound_filter::result_type thread_bound_filter::internal_process_item(bool
             return end_of_stream; 
         }
     } else { /* this is not an input filter */
-        while( !my_input_buffer->return_item(info, /*advance=*/true) ) {
-            if( my_pipeline->end_of_input && my_input_buffer->low_token == my_pipeline->token_counter )
-                return end_of_stream;
-            if( is_blocking )
-                __TBB_Yield();
-            else
+        while(!my_input_buffer->has_item()) {
+            if(!is_blocking) {
                 return item_not_available;
+            }
+            my_input_buffer->sema_P();
+            if( my_pipeline->end_of_input && !has_more_work()) {
+                return end_of_stream;
+            }
+        }
+        if(!my_input_buffer->return_item(info, /*advance*/true)) {
+            __TBB_ASSERT(0,"return_item failed");
         }
         info.my_object = (*this)(info.my_object);
     }
     if( next_filter_in_pipeline ) {
-        next_filter_in_pipeline->my_input_buffer->put_item(info);
+        if (!next_filter_in_pipeline->my_input_buffer->put_token(info,/*force_put=*/true) ) {
+            __TBB_ASSERT(0, "Couldn't put token after thread-bound buffer");
+        }
     } else {
-        my_pipeline->input_tokens++;
+        size_t ntokens_avail = ++(my_pipeline->input_tokens);
+        if(my_pipeline->filter_list->is_bound()) {
+            if(ntokens_avail == 1) {
+                my_pipeline->filter_list->my_input_buffer->sema_V();
+            }
+        }
     }
 
     return success;
