@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -28,39 +28,81 @@
 
 // Declarations for rock-bottom simple test harness.
 // Just include this file to use it.
-// Every test is presumed to have a command line of the form "foo [-v] [nthread]"
-// The default for nthread is 2.
+// Every test is presumed to have a command line of the form "test [-v] [MinThreads[:MaxThreads]]"
+// The default for MinThreads is 1, for MaxThreads 4.
+// The defaults can be overridden by defining macros HARNESS_DEFAULT_MIN_THREADS
+// and HARNESS_DEFAULT_MAX_THREADS before including harness.h
 
 #ifndef tbb_tests_harness_H
 #define tbb_tests_harness_H
 
+#include "tbb/tbb_config.h"
+
+namespace Harness {
+    enum TestResult {
+        Done,
+        Skipped,
+        Unknown
+    };
+}
+
+//! Entry point to a TBB unit test application
+/** It MUST be defined by the test application.
+
+    If HARNESS_NO_PARSE_COMMAND_LINE macro was not explicitly set before including harness.h,
+    then global variables Verbose, MinThread, and MaxThread will be available and
+    initialized when it is called.
+
+    Returns Harness::Done when the tests passed successfully. When the test fail, it must
+    not return, calling exit(errcode) or abort() instead. When the test is not supported
+    for the given platform/compiler/etc, it should return Harness::Skipped.
+
+    To provide non-standard variant of main() for the test, define HARNESS_CUSTOM_MAIN
+    before including harness.h **/
+int TestMain ();
+
 #define __TBB_LAMBDAS_PRESENT  ( _MSC_VER >= 1600 && !__INTEL_COMPILER || __INTEL_COMPILER > 1100 && _TBB_CPP0X )
 
-#if __SUNPRO_CC
-#include <stdlib.h>
-#include <string.h>
+#if defined(_MSC_VER) && _MSC_VER < 1400
+    #define __TBB_EXCEPTION_TYPE_INFO_BROKEN 1
 #else
-#include <cstdlib>
-#include <cstring>
+    #define __TBB_EXCEPTION_TYPE_INFO_BROKEN 0
 #endif
+
+#if __SUNPRO_CC
+    #include <stdlib.h>
+    #include <string.h>
+#else /* !__SUNPRO_CC */
+    #include <cstdlib>
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+    #pragma warning (push)
+    #pragma warning (disable: 4530)
+#endif
+    #include <cstring>
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    #pragma warning (pop)
+#endif
+#endif /* !__SUNPRO_CC */
+
 #include <new>
 
-#if __LRB__
-#include "harness_lrb.h"
-#else
-#define __TBB_TEST_EXPORT
-#define REPORT_FATAL_ERROR REPORT
-#endif /* !__LRB__ */
+    #define HARNESS_EXPORT
+    #define REPORT_FATAL_ERROR REPORT
 
 #if _WIN32||_WIN64
-    #include <windows.h>
+    #include "tbb/machine/windows_api.h"
+#if _XBOX
+    #undef HARNESS_NO_PARSE_COMMAND_LINE
+    #define HARNESS_NO_PARSE_COMMAND_LINE 1
+#endif
     #include <process.h>
 #else
     #include <pthread.h>
 #endif
 #if __linux__
-#include <sys/utsname.h> /* for uname */
-#include <errno.h>       /* for use in LinuxKernelVersion() */
+    #include <sys/utsname.h> /* for uname */
+    #include <errno.h>       /* for use in LinuxKernelVersion() */
 #endif
 
 #include "harness_report.h"
@@ -69,7 +111,7 @@
 #include "harness_assert.h"
 
 typedef void (*test_error_extra_t)(void);
-static test_error_extra_t ErrorExtraCall; 
+static test_error_extra_t ErrorExtraCall;
 //! Set additional handler to process failed assertions
 void SetHarnessErrorProcessing( test_error_extra_t extra_call ) {
     ErrorExtraCall = extra_call;
@@ -84,13 +126,13 @@ void ReportError( const char* filename, int line, const char* expression, const
 #endif
     if( ErrorExtraCall )
         (*ErrorExtraCall)();
-#if TBB_TERMINATE_ON_ASSERT
+#if HARNESS_TERMINATE_ON_ASSERT
     TerminateProcess(GetCurrentProcess(), 1);
-#elif TBB_EXIT_ON_ASSERT
+#elif HARNESS_EXIT_ON_ASSERT
     exit(1);
 #else
     abort();
-#endif /* TBB_EXIT_ON_ASSERT */
+#endif /* HARNESS_EXIT_ON_ASSERT */
 }
 //! Reports warnings issued by failed warning assertions
 void ReportWarning( const char* filename, int line, const char* expression, const char * message ) {
@@ -102,27 +144,36 @@ void ReportWarning( const char* filename, int line, const char* expression, cons
 #endif /* HARNESS_NO_ASSERT */
 
 #if !HARNESS_NO_PARSE_COMMAND_LINE
-//! Controls level of commentary.
+
+//! Controls level of commentary printed via printf-like REMARK() macro.
 /** If true, makes the test print commentary.  If false, test should print "done" and nothing more. */
 static bool Verbose;
 
+#ifndef HARNESS_DEFAULT_MIN_THREADS
+    #define HARNESS_DEFAULT_MIN_THREADS 1
+#endif
+
 //! Minimum number of threads
-/** The default is 0, which is typically interpreted by tests as "run without TBB". */
-static int MinThread = 0;
+static int MinThread = HARNESS_DEFAULT_MIN_THREADS;
+
+#ifndef HARNESS_DEFAULT_MAX_THREADS
+    #define HARNESS_DEFAULT_MAX_THREADS 4
+#endif
 
 //! Maximum number of threads
-static int MaxThread = 2;
+static int MaxThread = HARNESS_DEFAULT_MAX_THREADS;
 
-//! Parse command line of the form "name [-v] [nthread]"
+//! Parse command line of the form "name [-v] [MinThreads[:MaxThreads]]"
 /** Sets Verbose, MinThread, and MaxThread accordingly.
     The nthread argument can be a single number or a range of the form m:n.
-    A single number m is interpreted as if written m:m. 
-    The numbers must be non-negative.  
+    A single number m is interpreted as if written m:m.
+    The numbers must be non-negative.
     Clients often treat the value 0 as "run sequentially." */
 static void ParseCommandLine( int argc, char* argv[] ) {
-    int i = 1;  
+    if( !argc ) REPORT("Command line with 0 arguments\n");
+    int i = 1;
     if( i<argc ) {
-        if( strcmp( argv[i], "-v" )==0 ) {
+        if( strncmp( argv[i], "-v", 2 )==0 ) {
             Verbose = true;
             ++i;
         }
@@ -132,12 +183,12 @@ static void ParseCommandLine( int argc, char* argv[] ) {
         MinThread = strtol( argv[i], &endptr, 0 );
         if( *endptr==':' )
             MaxThread = strtol( endptr+1, &endptr, 0 );
-        else if( *endptr=='\0' ) 
+        else if( *endptr=='\0' )
             MaxThread = MinThread;
         if( *endptr!='\0' ) {
             REPORT_FATAL_ERROR("garbled nthread range\n");
             exit(1);
-        }    
+        }
         if( MinThread<0 ) {
             REPORT_FATAL_ERROR("nthread must be nonnegative\n");
             exit(1);
@@ -163,6 +214,25 @@ static void ParseCommandLine( int argc, char* argv[] ) {
 }
 #endif /* HARNESS_NO_PARSE_COMMAND_LINE */
 
+#if !HARNESS_CUSTOM_MAIN
+
+
+HARNESS_EXPORT
+#if HARNESS_NO_PARSE_COMMAND_LINE
+int main() {
+#else
+int main(int argc, char* argv[]) {
+    ParseCommandLine( argc, argv );
+#endif
+    int res = TestMain ();
+    ASSERT( res==Harness::Done || res==Harness::Skipped, "Wrong return code by TestMain");
+    REPORT( res==Harness::Done ? "done\n" : "skip\n" );
+    return 0;
+}
+
+
+#endif /* !HARNESS_CUSTOM_MAIN */
+
 //! Base class for prohibiting compiler-generated operator=
 class NoAssign {
     //! Assignment not allowed
@@ -176,7 +246,7 @@ public:
 
 //! Base class for prohibiting compiler-generated copy constructor or operator=
 class NoCopy: NoAssign {
-    //! Copy construction not allowed  
+    //! Copy construction not allowed
     NoCopy( const NoCopy& );
 public:
     NoCopy() {}
@@ -202,9 +272,9 @@ public:
     #pragma warning (push)
     #pragma warning (disable: 2193)
 #endif /* __ICC==1100 */
-        // Some machines may have very large hard stack limit. When the test is 
-        // launched by make, the default stack size is set to the hard limit, and 
-        // calls to pthread_create fail with out-of-memory error. 
+        // Some machines may have very large hard stack limit. When the test is
+        // launched by make, the default stack size is set to the hard limit, and
+        // calls to pthread_create fail with out-of-memory error.
         // Therefore we set the stack size explicitly (as for TBB worker threads).
         const size_t MByte = 1<<20;
 #if __i386__||__i386
@@ -237,7 +307,7 @@ public:
 #else
         int status = pthread_join( thread_id, NULL );
         ASSERT( !status, "pthread_join failed" );
-#endif 
+#endif
     }
 
 private:
@@ -260,15 +330,7 @@ private:
 #endif
     {
         NativeParallelForTask& self = *static_cast<NativeParallelForTask*>(object);
-#if defined(__EXCEPTIONS) || defined(_CPPUNWIND) || defined(__SUNPRO_CC)
-        try {
-            (self.body)(self.index);
-        } catch(...) {
-            ASSERT( false, "uncaught exception" );
-        }
-#else
         (self.body)(self.index);
-#endif// exceptions are enabled
         return 0;
     }
 };
@@ -284,7 +346,7 @@ void NativeParallelFor( Index n, const Body& body ) {
         task* array = static_cast<task*>(operator new( n*sizeof(task) ));
 
         // Construct the tasks
-        for( Index i=0; i!=n; ++i ) 
+        for( Index i=0; i!=n; ++i )
             new( &array[i] ) task(i,body);
 
         // Start the tasks
@@ -304,47 +366,52 @@ void NativeParallelFor( Index n, const Body& body ) {
 
 //! The function to zero-initialize arrays; useful to avoid warnings
 template <typename T>
-void zero_fill(void* array, size_t N) {
-    memset(array, 0, sizeof(T)*N);
+void zero_fill(void* array, size_t n) {
+    memset(array, 0, sizeof(T)*n);
 }
 
+#if __SUNPRO_CC && defined(min)
+#undef min
+#undef max
+#endif
+
 #ifndef min
-    //! Utility template function returning lesser of the two values.
-    /** Provided here to avoid including not strict safe <algorithm>.\n
-        In case operands cause signed/unsigned or size mismatch warnings it is caller's
-        responsibility to do the appropriate cast before calling the function. **/
-    template<typename T1, typename T2>
-    T1 min ( const T1& val1, const T2& val2 ) {
-        return val1 < val2 ? val1 : val2;
-    }
+//! Utility template function returning lesser of the two values.
+/** Provided here to avoid including not strict safe <algorithm>.\n
+    In case operands cause signed/unsigned or size mismatch warnings it is caller's
+    responsibility to do the appropriate cast before calling the function. **/
+template<typename T1, typename T2>
+T1 min ( const T1& val1, const T2& val2 ) {
+    return val1 < val2 ? val1 : val2;
+}
 #endif /* !min */
 
 #ifndef max
-    //! Utility template function returning greater of the two values. Provided here to avoid including not strict safe <algorithm>.
-    /** Provided here to avoid including not strict safe <algorithm>.\n
-        In case operands cause signed/unsigned or size mismatch warnings it is caller's
-        responsibility to do the appropriate cast before calling the function. **/
-    template<typename T1, typename T2>
-    T1 max ( const T1& val1, const T2& val2 ) {
-        return val1 < val2 ? val2 : val1;
-    }
+//! Utility template function returning greater of the two values.
+/** Provided here to avoid including not strict safe <algorithm>.\n
+    In case operands cause signed/unsigned or size mismatch warnings it is caller's
+    responsibility to do the appropriate cast before calling the function. **/
+template<typename T1, typename T2>
+T1 max ( const T1& val1, const T2& val2 ) {
+    return val1 < val2 ? val2 : val1;
+}
 #endif /* !max */
 
 #if __linux__
 inline unsigned LinuxKernelVersion()
 {
-    unsigned a, b, c;
+    unsigned digit1, digit2, digit3;
     struct utsname utsnameBuf;
-    
+
     if (-1 == uname(&utsnameBuf)) {
         REPORT_FATAL_ERROR("Can't call uname: errno %d\n", errno);
         exit(1);
     }
-    if (3 != sscanf(utsnameBuf.release, "%u.%u.%u", &a, &b, &c)) {
+    if (3 != sscanf(utsnameBuf.release, "%u.%u.%u", &digit1, &digit2, &digit3)) {
         REPORT_FATAL_ERROR("Unable to parse OS release '%s'\n", utsnameBuf.release);
         exit(1);
     }
-    return 1000000*a+1000*b+c;
+    return 1000000*digit1+1000*digit2+digit3;
 }
 #endif
 
@@ -384,12 +451,20 @@ public:
 
 #if _WIN32 || _WIN64
     void Sleep ( int ms ) { ::Sleep(ms); }
+
+    typedef DWORD tid_t;
+    tid_t CurrentTid () { return GetCurrentThreadId(); }
+
 #else /* !WIN */
+
     void Sleep ( int ms ) {
         timespec  requested = { ms / 1000, (ms % 1000)*1000000 };
-        timespec  remaining = {0};
+        timespec  remaining = { 0, 0 };
         nanosleep(&requested, &remaining);
     }
+
+    typedef pthread_t tid_t;
+    tid_t CurrentTid () { return pthread_self(); }
 #endif /* !WIN */
 
 } // namespace Harness
