@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -63,12 +63,22 @@ void TestMap() {
         ASSERT( m.find(i)->second==i*i, NULL );
 }
 
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+    #pragma warning (push)
+    #pragma warning (disable: 4530)
+#endif
+
 #include <deque>
 #include <list>
 #include <map>
 #include <set>
 #include <vector>
 
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    #pragma warning (pop)
+#endif
+
 template<typename Allocator>
 void TestAllocatorWithSTL() {
     typedef typename Allocator::template rebind<int>::other Ai;
