@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -90,8 +90,8 @@ static void SerialTest() {
         }
 }
 
-#include "harness.h"
 #include "tbb/parallel_for.h"
+#include "harness.h"
 
 const int N = 1<<22;
 
@@ -118,14 +118,11 @@ void ParallelTest() {
 
 #include "tbb/task_scheduler_init.h"
 
-__TBB_TEST_EXPORT
-int main( int argc, char* argv[] ) {
-    ParseCommandLine(argc,argv);
+int TestMain () {
     SerialTest();
     for( int p=MinThread; p<=MaxThread; ++p ) {
         tbb::task_scheduler_init init(p);
         ParallelTest();
     }
-    REPORT("done\n");
-    return 0;
+    return Harness::Done;
 }
