@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -37,6 +37,14 @@
 #if MALLOC_REPLACEMENT_AVAILABLE
 
 #if _WIN32 || _WIN64
+// As the test is intentionally build with /EHs-, suppress multiple VS2005's 
+// warnings like C4530: C++ exception handler used, but unwind semantics are not enabled
+#if defined(_MSC_VER) && !__INTEL_COMPILER
+/* ICC 10.1 and 11.0 generates code that uses std::_Raise_handler,
+   but it's only defined in libcpmt(d), which the test doesn't linked with.
+ */
+#define _HAS_EXCEPTIONS 0
+#endif
 // to use strdup and putenv w/o warnings
 #define _CRT_NONSTDC_NO_DEPRECATE 1
 #endif
@@ -58,6 +66,7 @@
 #if __MINGW32__
 #include <unistd.h>
 #else
+typedef unsigned __int16 uint16_t;
 typedef unsigned __int32 uint32_t;
 typedef unsigned __int64 uint64_t;
 #endif
@@ -68,6 +77,13 @@ typedef unsigned __int64 uint64_t;
 // On Windows, the tricky way to print "done" is necessary to create 
 // dependence on msvcpXX.dll, for sake of a regression test.
 // On Linux, C++ RTL headers are undesirable because of breaking strict ANSI mode.
+#if defined(_MSC_VER) && _MSC_VER >= 1300 && _MSC_VER <= 1310 && !defined(__INTEL_COMPILER)
+/* Fixing compilation error reported by VS2003 for exception class
+   when _HAS_EXCEPTIONS is 0: 
+   bad_cast that inherited from exception is not in std namespace.
+*/
+using namespace std;
+#endif
 #include <string>
 #endif
 
@@ -83,17 +99,38 @@ static inline bool isAligned(T arg, uintptr_t alignment) {
 
 /* Below is part of MemoryAllocator.cpp. */
 
-/*
- * The identifier to make sure that memory is allocated by scalable_malloc.
- */
-const uint64_t theMallocUniqueID=0xE3C7AF89A1E2D8C1ULL; 
+class BackRefIdx { // composite index to backreference array
+private:
+    uint16_t master;      // index in BackRefMaster
+    uint16_t largeObj:1;  // is this object "large"?
+    uint16_t offset  :15; // offset from beginning of BackRefBlock
+public:
+    BackRefIdx() : master((uint16_t)-1) {}
+    bool isInvalid() { return master == (uint16_t)-1; }
+    bool isLargeObject() const { return largeObj; }
+    uint16_t getMaster() const { return master; }
+    uint16_t getOffset() const { return offset; }
 
-struct LargeObjectHeader {
-    void        *unalignedResult;   /* The base of the memory returned from getMemory, this is what is used to return this to the OS */
-    size_t       unalignedSize;     /* The size that was requested from getMemory */
-    uint64_t     mallocUniqueID;    /* The field to check whether the memory was allocated by scalable_malloc */
-    size_t       objectSize;        /* The size originally requested by a client */
-    bool         getMemoryBlock;    /* Is memory allocated by getMemory or by getRawMemory? */
+    // only newBackRef can modify BackRefIdx
+    static BackRefIdx newBackRef(bool largeObj);
+};
+
+struct LargeMemoryBlock {
+    LargeMemoryBlock *next,          // ptrs in list of cached blocks
+                     *prev;
+    uintptr_t         age;           // age of block while in cache
+    size_t            objectSize;    // the size requested by a client
+    size_t            unalignedSize; // the size requested from getMemory
+    bool              fromMapMemory;
+    BackRefIdx        backRefIdx;    // cached here, used copy is in LargeObjectHdr
+};
+
+struct LargeObjectHdr {
+    LargeMemoryBlock *memoryBlock;
+    /* Have to duplicate it here from CachedObjectHdr, 
+       as backreference must be checked without further pointer dereference.
+       Points to LargeObjectHdr. */
+    BackRefIdx       backRefIdx;
 };
 
 /*
@@ -112,8 +149,8 @@ static bool scalableMallocLargeBlock(void *object, size_t size)
     ASSERT(_msize(object) >= size, NULL);
 #endif
 
-    LargeObjectHeader *h = (LargeObjectHeader*)((uintptr_t)object-sizeof(LargeObjectHeader));
-    return h->mallocUniqueID==theMallocUniqueID && h->objectSize==size;
+    LargeMemoryBlock *lmb = ((LargeObjectHdr*)object-1)->memoryBlock;
+    return uintptr_t(lmb)<uintptr_t(((LargeObjectHdr*)object-1)) && lmb->objectSize==size;
 }
 
 struct BigStruct {
@@ -226,6 +263,7 @@ int main(int , char *[]) {
 }
 
 #define HARNESS_NO_PARSE_COMMAND_LINE 1
+#define HARNESS_CUSTOM_MAIN 1
 #include "harness.h"
 
 #else  /* !MALLOC_REPLACEMENT_AVAILABLE */
